namespace de.osci.osci12.common {

	using System;
	using System.Globalization;
	using org.apache.commons.logging;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;
	using java.util;

	/// <summary> Der DialogHandler steuert die Kommunikation mit dem Intermedir. 
	/// Fr die Kommunikation mit dem Intermedir mssen eine Reihe von Rahmenparametern
	/// gesetzt werden. Daher ist diese Klasse <b>zentral</b> fr jede Kommunikation. 
	/// Ein DialogHandler-Objekt ist fr jede Nachricht erforderlich, unabhngig davon, 
	/// ob diese innerhalb eines impliziten oder expliziten Dialogs verarbeitet wird.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class DialogHandler {


		private void  InitBlock() 
			{
			secureRandomAlgorithm = Constants.SECURE_RANDOM_ALGORITHM_SHA1;
			controlblock = new ControlBlockH();
			CultureInfo cultureInfo = System.Globalization.CultureInfo.CurrentCulture;
			languageList = cultureInfo.TwoLetterISOLanguageName;
            symmetricCipherAlgorithm = de.osci.osci12.common.Constants.SYMMETRIC_CIPHER_ALGORITHM_AES256;
            }

		/// <summary> Liefert eine Wert, der festlegt, ob Nachrichtensignaturen (Client-/Suppliersignaturen)
		/// beim Empfang geprft werden sollen oder legt ihn fest. Voreinstellung ist <b>true</b>. 
		/// <p><b>Achtung: </b>Diese Eigenschaft legt lediglich fest, ob vorhandene
		/// Signaturen eingehender Nachrichten mathematisch geprft werden. Fehlt eine
		/// Signatur ganz, wird keine Exception o.. ausgelst. Es sollte daher
		/// zustzlich OSCIMessage.isSigned() geprft werden.</p>
		/// </summary>
		/// <value><b>true</b> - Signaturprfung wird durchgefhrt.
		/// </value>
		virtual public bool CheckSignatures {
			get {
				return checkSignatures;
			}			
			set {
				this.checkSignatures = value;
			}	
		}

		/// <summary> Liefert eine Wert, der festlegt, ob Nachrichtensignaturen (Client-/Suppliersignaturen)
		/// beim Versand angebracht werden sollen oder legt ihn fest. Die Voreinstellung ist <b>true</b>.
		/// </summary>
		/// <value><b>true</b> -> ausgehende Nachrichten werden signiert.
		/// </value>
		/// #isCreateSignatures()
		// <seealso cref="CreateSignatures()">
		// </seealso>
		virtual public bool CreateSignatures {
			get {
				return createSignatures;
			}			
			set {
				this.createSignatures = value;
			}			
		}

		
		/// <summary> Liefert die installierte Implementierung des Transportinterfaces.
		/// </summary>
		/// <value> Transportinterface-Implementierung.
		/// </value>
		/// <seealso cref="de.osci.osci12.extinterfaces.TransportI">
		/// </seealso>
		virtual public TransportI TransportModule {
			get {
				return transportModule;
			}	
		}

		
		/// <summary>Liefert eine Wert, der festlegt, ob die Nachrichten als verschlsselte Auftragsdaten versendet
		/// werden oder legt ihn fest. Voreinstellung ist <b>true</b>.
		/// </summary>
		/// <value><b>true</b> -> ausgehende Nachrichten werden verschlsselt.
		/// </value>
		virtual public bool Encryption {
			get {
				return encryption;
			}
			set {
				this.encryption = value;
			}
		}

		/// <summary> Liefert den aktuellen Controlblock. Ein Controlblock beinhaltet
		/// Challenge, Response, ConversationID und SequenzeNumber.
		/// </summary>
		/// <value> ControlBlock dieses Dialogs.
		/// </value>
		virtual public ControlBlockH Controlblock {
			get {
				return controlblock;
			}
		}
		
		/// <summary> Liefert die Instanz (einer Implementierung) des ProgressEventHandlerI-
		/// Interfaces oder setzt sie. Das registrierte Objekt wird im Verlauf der Verarbeitung der
		/// Nachricht durch Aufruf seiner Methode event(int,String,int) ber
		/// Fortschrittsereignisse informiert.
		/// </summary>
		/// <value>ProgressEventHandlerI-Implementierung
		/// </value>
		virtual public ProgressEventHandlerI ProgressEventHandler {
			get {
				return progressEventHandler;
			}
			set {
				this.progressEventHandler = value;
			}
		}

		/// <summary> Registriert eine Instanz (einer Implementierung) der abstrakten Klasse
		/// DialogFinder zur Verwendung in diesem Dialog. Voreingestellt ist eine Instanz
		/// der Default-Implementierung de.osci.osci12.extinterfaces.DialogFinder.
		/// </summary>
		/// <value>dialogFinder
		/// </value>
		/// <seealso cref="de.osci.osci12.extinterfaces.DialogFinder">
		/// </seealso>
		public static de.osci.osci12.extinterfaces.DialogFinder DialogFinder {
			set {
				dialogFinder = value;
			}	
		}

		/// <summary> Liefert den mit diesem DialogHandler verbundenen Client.
		/// </summary>
		/// <value> das Rollenobjekt, welches als Client fungiert.
		/// </value>
		virtual public Role Client {
			get {
				return this.client;
			}			
		}

		/// <summary> Liefert den mit diesem DialogHandler verbundenen Supplier.
		/// </summary>
		/// <value> das Rollenobjekt, welches als Supplier fungiert
		/// </value>
		virtual public Role Supplier {
			get {
				return supplier;
			}			
		}
	
		/// <summary> Liefert die Liste der Sprachkrzel, die in den DesiredLanguages-Elementen
		/// eingetragen wird oder setzt sie. Voreingestellt ist das Krzel der im default-CurrentCulture
		/// eingetragenen Sprache.
		/// </summary>
		/// <value>Liste der Sprachkrzel, getrennt durch Leerzeichen, z.B. "de en-US fr".
		/// </value>
		virtual public System.String LanguageList {
			get {
				return languageList;
			}			
			set {
				this.languageList = value;
			}	
		}

		/// <summary> Liefert den eingestellten Algorithmus fr den verwendeten
		/// Zufallszahlengenerator oder setzt ihn. Dieser String wird von der Bibliothek bei der
		/// Initialisierung an die Methode java.security.SecureRandom#getInstance(String)
		/// bergeben. Voreingestellt ist "SHA1PRNG".
		/// </summary>
		/// <value>String-Identifier fr den Algorithmus, der von dem installierten
		/// Provider untersttzt werden mu.
		/// </value>
		// <seealso cref="">java.security.SecureRandom
		// </seealso>
		virtual public System.String SecureRandomAlgorithm {
			get {
				return secureRandomAlgorithm;
			}			
			set {
				this.secureRandomAlgorithm = value;
			}
		}

		/// <summary> Liefert/Setzt den Identifier des Signaturalgorithmus, der fr die
		/// Signatur der Nachrichten (verschlsselte Auftragsdaten) verwendet wird, sofern
        /// die Signer-Implementierung keinen anderen Algorithmus verlangt.
		/// </summary>
		/// <value> Identifier
		/// </value>
		/// #setSignatureAlgorithm(String)
		/// <seealso cref="SignatureAlgorithm()">
		/// </seealso>
		static public System.String SignatureAlgorithm {
			get {
				return signatureAlgorithm;
			}
            set
            {
                signatureAlgorithm = value;
            }
		}

		/// <summary><p>Hiermit kann eine Implementierung der abstrakten Klasse OSCIDataSource
		/// installiert werden, falls Inhaltsdaten nicht durch die default-Implementierung
		/// SwapBuffer im Arbeitsspeicher bzw. in temporren Dateien gepuffert werden
		/// sollen, sondern beispielsweise in einer Datenbank.</p>
		/// Dieser Puffer-Mechanismus wird von den Klassen EncryptedData, Content und Attachment
		/// genutzt. Zur Implementierung eigener Klassen sind die Hinweise in der
		/// Dokumentation von OSCIDataSource zu beachten.
		/// </summary>
		/// <seealso cref="OSCIDataSource">
		/// </seealso>
		/// <seealso cref="SwapBuffer">
		/// </seealso>
		/// <value>die OSCIDataSource-Implementierung
		/// </value>
		public static OSCIDataSource DataBuffer {
			set {
				dataBuffer = value;
			}	
		}

		/// <summary> Interne Methode, sollte von Anwendungen nicht aufgerufen werden.
		/// </summary>
		public static OSCIDataSource NewDataBuffer {
			get {
				return dataBuffer.newInstance();
			}			
		}

		/// <summary> Diese Eigenschaft ist noch fr die Abwrtskompatibilitt vorhanden,
		/// wurde ersetzt durch DefaultSuppliers
		/// </summary>
		/// <value>Rollenobjekt
		/// </value>
		public static Addressee DefaultSupplier 
		{
			set 
			{
				defaultSupplier = new Addressee[1];
				defaultSupplier[0] = value;
			}
			get 
			{
				if (defaultSupplier == null)
					throw new Exception("No default supplier configured.");
				return (Addressee) defaultSupplier[0];
			}	
		}

		/// <summary> Passive Empfnger mssen Addressee-Objekte als Default-Supplier setzen, 
		/// die fr den Empfang von Nachrichten die richtigen Decrypter- und gegebenenfalls 
		/// Signer-Objekte mit den Privatschlsseln zur Verfgung stellen.
		/// </summary>
		/// <value>Rollenobjekte
		/// </value>
		public static Addressee[] DefaultSuppliers 
		{
			set 
			{
				defaultSupplier = value;
			}
			get 
			{
				return defaultSupplier;
			}	
		}

		private System.String symmetricCipherAlgorithm;

		/// <summary> Liefert den symmetrischen Verschlsselungs-Algorithmus.
		/// </summary>
		/// <value>Identifier des Algorithmus
		/// </value>
		public System.String SymmetricCipherAlgorithm {
			get {
				return symmetricCipherAlgorithm;
			}
			set {
				symmetricCipherAlgorithm = value;
			}
		}

		/// <summary> Liefert den Hash-Algorithmus fr die Signatur der Nachrichten 
		/// oder setzt ihn fest (Voreinstellung SHA256).
		/// </summary>
		/// <value>Identifier des Hash-Algorithmus
		/// </value>
		public static System.String DigestAlgorithm {
			get {
				return digestAlgorithm;
			}			
			set {
				digestAlgorithm = value;
			}
		}
		
		/// <summary>ResourceBundle-Objekt, welches bibliotheksweit fr die Textausgaben verwendet wird.</summary>
		public static ResourceBundle text;
		
		/// <summary>Hashalgortihmus (Voreinstellung SHA256)</summary>
		private static System.String digestAlgorithm;

		private static Log log;
		
		/// <summary>Zufallsgenerator-Algorithmus </summary>
		private System.String secureRandomAlgorithm;

		/// <summary>Signaturalgorithmus (Nachrichtensignatur) </summary>
		private static System.String signatureAlgorithm;

		/// <summary>Signatur des Suppliers prfen.</summary>
		private bool checkSignatures = true;

		/// <summary>Der Controlblock.</summary>
		private ControlBlockH controlblock;

		/// <summary>Nachrichtensignatur anbringen.</summary>
		private bool createSignatures = true;

		/// <summary>Ein Objekt welches das Interface TranportI implementiert. </summary>
		private TransportI transportModule;

		/// <summary>Nachrichtenverschluesselung</summary>
		private bool encryption = true;

		/// <summary>Sprachliste</summary>
		private System.String languageList;

		/// <summary>Implementierung des Interfaces ProgressEventHandlerI.</summary>
		private ProgressEventHandlerI progressEventHandler = null;

		/// <summary>Client</summary>
		public Role client = null;

		/// <summary>Supplier</summary>
		public Role supplier = null;

		/// <summary>For internal use only </summary>
		public static de.osci.osci12.extinterfaces.DialogFinder dialogFinder;

		/// <summary>Der Trenner fr die einzelnen MIME boundaries.</summary>
		public static System.String boundary = "MIME_boundary";

		// CipherCert immer mitschicken. Auch wenn nicht verschlsselt wird.
		// Diese Var wird nur beim Intermedir ausgewertet
		/// <summary> Status impliziter/expliziter Dialog. Sollte von Anwendungen nicht gesetzt werden.
		/// </summary>
		public bool explicitDialog = false;

		private static OSCIDataSource dataBuffer;		
		
		// Hier merkt sich der Intermedir nach dem Abholen eines Zustellauftrags
		// dessen Laufzettel, um bei einem Nachfolgeauftrag den Zeitstempel "reception"
		// zu ergnzen.
		/// <summary> For internal use only
		/// </summary>
		public static Addressee[] defaultSupplier;

		public de.osci.osci12.messageparts.ProcessCardBundle processCardForTimestampReception;
		
		/// <summary> DialogHandler fr alle Auftrge/Auftragsantworten,
		/// ausgenommen Annahmeauftrag/-antwort und Bearbeitungsauftrag/-antwort.
		/// </summary>
		/// <param name="client">Sollen Auftrge signiert versendet werden, 
		/// oder werden verschlsselte Rckantworten erwartet, so muss fr dieses
		/// Objekt ein Decrypter- bzw. Signer-Objekt gesetzt sein. Dies gilt fr die
		/// Verschlsselung bzw. Signatur der Nachricht wie fr die der Inhaltsdaten.
		/// </param>
		/// <param name="supplier">Intermediaer als Supplier.
		/// </param>
		/// <param name="transportModule">zu verwendende Implementierung des TransportInterfaces.
		/// </param>
		/// <seealso cref="de.osci.osci12.roles.Originator">
		/// </seealso>
		public DialogHandler(Originator client, Intermed supplier, TransportI transportModule) {
			InitBlock();
			this.supplier = supplier;
			if ((supplier == null) && (defaultSupplier != null))
			{
				this.supplier = defaultSupplier[0];
			}
			this.transportModule = transportModule;
			this.client = client;
		}
		
		/// <summary>  DialogHandler fr Annahmeauftrag/-antwort und Bearbeitungsauftrag/-antwort.
		/// Dieser Konstruktor wird vom Anwender nicht bentigt.
		/// </summary>
		/// <param name="client">Intermedir als Client.
		/// </param>
		/// <param name="supplier">Addressee als Supplier.
		/// </param>
		/// <param name="transportModule">Implementierung des TransportInterfaces
		/// </param>
		/// <seealso cref="de.osci.osci12.roles.Addressee">
		/// </seealso>
		//   @todo muss hier im Konstruktor ein Transportmodul bergeben werden
		public DialogHandler(Intermed client, Addressee supplier, TransportI transportModule) {
			InitBlock();
			this.supplier = supplier;
			if ((supplier == null) && (defaultSupplier != null))
			{
				this.supplier = defaultSupplier[0];
			}
			this.transportModule = transportModule;
			this.client = client;
		}
		
		/// <summary>Speichert den Challenge-Wert einer vorangegangenen Response-Nachricht.
		/// Wird von Anwendungen normalerweise nicht bentigt und sollte auch nicht gesetzt werden.
		/// </summary>
		public System.String prevChallenge;
		

		/// <summary> Interne Methode, sollte von Anwendungen nicht aufgerufen werden.
		/// </summary>
		public virtual void  checkControlBlock(ControlBlockH cb) {			
			log.debug("\n!!!!!!!RSP!!!!!!!!! " + controlblock.Response + " : " + cb.Response);
			log.debug("\n!!!!!!!CHALL!!!!!!!!! " + controlblock.Challenge + " : " + cb.Challenge);
			log.debug("\n!!!!!!!ConvID!!!!!!!!! " + controlblock.ConversationID + " : " + cb.ConversationID);
			log.debug("\n!!!!!!!SQU!!!!!!!!! " + controlblock.SequenceNumber + " : " + cb.SequenceNumber);
			
			if (controlblock.Challenge != null) {
				if (!cb.Response.Equals(controlblock.Challenge)) {
					throw new OSCIErrorException("9400");
				}
			}
			
			prevChallenge = cb.Challenge;
			
			if (controlblock.ConversationID != null) {
				// der folgende Fall kann eigentlich nicht eintreten, weil der ControlBlock anhand der ConvId gesucht wurde
				if (!controlblock.ConversationID.Equals(cb.ConversationID)) {
					throw new OSCIErrorException("9400");
				}
			} else {
				controlblock.ConversationID = cb.ConversationID;
			}
			if ((cb.SequenceNumber > - 1) && (cb.SequenceNumber != controlblock.SequenceNumber)) {
				throw new OSCIErrorException("9400");
			}
			
			controlblock.SequenceNumber = cb.SequenceNumber;
			controlblock.Challenge = cb.Challenge;
			controlblock.Response = cb.Response;
		}

		/// <summary>Liefert den registrierten Security-Provider fr die kryptographischen
		/// Operationen.
		/// </summary>
		/// <value>Security-Provider
		/// </value>
		/// 

		/*
		* Diese Methode liefert den in einem Dialog schon benutzen DialogHandler.
		* @para controlBlock Control-Block nach dem der evt. schon bestehende Dialog gefunden werden soll.
		* Hierbei wird als erstes ein bestehender expliziter Dialog d.h. ConversationId ist vorhanden
		* kontrolliert ansonsten eine Bestehender Response Wert untersucht.
		* @return  Sollte es sich um einen expliziten/ oder impliziten  Dialog handeln wird der alte DialogHandler zurckgegeben
		* ansonsten this
		*/
		/// <summary> Interne Methode, sollte von Anwendungen nicht aufgerufen werden.
		/// </summary>
		// <param name="controlBlock">Control-Block
		// </param>
		// <returns>DialogHandler</returns>
		public static DialogHandler findDialog(ControlBlockH controlBlock) 
		{
			DialogHandler dh = null;
			if (dialogFinder != null) {
				dh = dialogFinder.findDialog(controlBlock);
			}
			if (dh == null) {
				throw new OSCIErrorException("9400");
			} else {
				return dh;
			}
		}
		
		/// <summary> Setzt den ControlBlock zurck. Erlaubt die Wiederverwendung dieses
		/// Objekts in einem neuen Dialog.
		/// </summary>
		public virtual void  resetControlBlock() {
			this.controlblock = new ControlBlockH();
		}
						
		static DialogHandler() 
		{ 
			text = ResourceBundle.getBundle("de.osci.osci12.extinterfaces.language.Text", System.Globalization.CultureInfo.CurrentCulture);
			digestAlgorithm = de.osci.osci12.common.Constants.DIGEST_ALGORITHM_SHA256;
			log = LogFactory.getLog(typeof(DialogHandler));
            signatureAlgorithm = Constants.SIGNATURE_ALGORITHM_RSA_SHA256;
            dataBuffer = new SwapBuffer();
		}
	}
}