﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;

namespace de.osci.osci12.dotcommon
{
    /// <summary>
    /// This class circumvents some strange en-/decryption problems.
    /// </summary>
    public sealed class RSA_Cipher : RSA
    {
        /// <summary>
        /// RSA_Cipher Constructor.
        /// </summary>
        /// <param name="rsaParameters">RSAParameter to import.</param>
        public RSA_Cipher(SupportClass.PrivateKeySupport pKeyInfo)
        {
            CspParameters csp = new CspParameters();
            csp.KeyContainerName = pKeyInfo.containerName;
            csp.ProviderName = pKeyInfo.providerName;
            csp.ProviderType = pKeyInfo.providerType;
            csp.KeyNumber = pKeyInfo.keySpec;
            if (pKeyInfo.isMachineKeyset)
                csp.Flags = CspProviderFlags.UseMachineKeyStore;

            ImportParameters(new RSACryptoServiceProvider(csp).ExportParameters(true));
        }

        public RSA_Cipher(java.security.cert.X509Certificate cert)
        {
            X509Certificate2 netCert = new X509Certificate2(cert.GetRawCertData());

            if (!(netCert.PublicKey.Key is RSACryptoServiceProvider))
                throw new Exception("Certificate must support RSA.");

            _rsaCSP = new RSACryptoServiceProvider();
            ImportParameters(((RSACryptoServiceProvider)netCert.PublicKey.Key).ExportParameters(false));
        }

        /// <summary>
        /// Current imported RSA-Parameters.
        /// </summary>
        private Nullable<RSAParameters> _rsaParams;
        private RSACryptoServiceProvider _rsaCSP = null;


        /// <summary>
        /// RSA-Parameters, which should be used
        /// </summary>
        public Nullable<RSAParameters> RSAParameters
        {
            get { return _rsaParams; }
        }

        private RSACryptoServiceProvider CryptoServiceProvider
        {
            get
            {
                if (_rsaCSP == null)
                {
                    lock (this)
                    {
                        if (_rsaCSP == null)
                        {
                            if (_rsaParams == null)
                            {
                                throw new ArgumentNullException("RSAParameters");
                            }

                            _rsaCSP = new RSACryptoServiceProvider();
                            _rsaCSP.ImportParameters(_rsaParams.Value);
                        }
                    }
                }

                return _rsaCSP;
            }
        }


        /// <summary>
        /// Decrypt the given byte-array by using the currently imported RSA-Parameters.
        /// </summary>
        /// <param name="rgb">Data to decrypt as byte array.</param>
        /// <returns></returns>
        public override byte[] DecryptValue(byte[] rgb)
        {
            return CryptoServiceProvider.Decrypt(rgb, false);
        }



        /// <summary>
        /// Encrypt the given byte-array by using the currently imported RSA-Parameters.
        /// </summary>
        /// <param name="rgb">Data to encrypt as byte array.</param>
        /// <returns></returns>
        public override byte[] EncryptValue(byte[] rgb)
        {
            return CryptoServiceProvider.Encrypt(rgb, false);
        }

        /// <summary>
        /// Overridden. Exports the RSAParameters.
        /// </summary>
        /// <param name="includePrivateParameters">true to include private parameters; otherwise, false. </param>
        /// <returns>The parameters for RSA.</returns>
        public override RSAParameters ExportParameters(bool includePrivateParameters)
        {
            throw new NotSupportedException();
        }
        
        /// <summary>
        /// Overridden. Imports the specified RSAParameters.
        /// </summary>
        /// <param name="parameters">The parameters for RSA.</param>
        public override void ImportParameters(RSAParameters parameters)
        {
            if (parameters.Modulus == null || parameters.Exponent == null)
            {
//                throw new Exception(ErrorText.InvalidRSAParams);
                throw new Exception("BUMM");
            }

            _rsaParams = parameters;
            _rsaCSP = null;
        }

        /// <summary>
        /// Dispose method.
        /// </summary>
        /// <param name="disposing"></param>
        protected override void Dispose(bool disposing)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        public override string KeyExchangeAlgorithm
        {
            get { return CryptoServiceProvider.KeyExchangeAlgorithm; }
        }
        
        /// <summary>
        /// 
        /// </summary>
        public override string SignatureAlgorithm
        {
            get 
            { 
                return CryptoServiceProvider.SignatureAlgorithm; 
            }
        }
    }
}
