namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.encryption;
	using de.osci.osci12.messagetypes;
	using de.osci.osci12.roles;
	using de.osci.osci12.signature;

	/// <summary> <p>Die EncryptedDataOSCI-Klasse stellt einen Datencontainer fr verschlsselte
	/// Daten in einer OSCI-Nachricht dar. Ein EncryptedDataOSCI-Objekt wird in eine
	/// OSCI-Nachricht eingestellt oder in einen Content-Conatiner(bei Merhrfachverschlsselung)
	/// und  kann selbst entweder ein Content-Container enthalten (Normalfall) oder ein
	/// EncryptedData-Objekt (fr den Parser).</p>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="ContentContainer">
	/// </seealso>
	/// <seealso cref="Attachment">
	/// </seealso>
	public class EncryptedDataOSCI:MessagePart {

		new private static Log log;
		private static int idNr = -1;
		 private int keyIDs = 0;
		private string ns;
		/// <summary> Das EncryptedData Objekt hlt Attachment Daten
		/// </summary>
		public static int TYPE_OF_DATA_ATTACHMENT = 0;

		/// <summary> Das EncryptedData Objekt hlt Content-Container Daten
		/// </summary>
		public static int TYPE_OF_DATA_CONTENTCONTAINER = 1;

		/// <summary> Das EncryptedData Objekt hlt eine OSCI MSG (Transportverschlsselung)
		/// </summary>
		public static int TYPE_OF_DATA_OSCI_MSG = 2;

		/// <summary> Das EncryptedData Objekt beinhaltet ein EncryptedData Objekt (wird vom Parser benutzt oder bei Mehrfachverschlsselung)
		/// </summary>
		public static int TYPE_OF_DATA_ENC_DATA = 3;

		private int typeOfData = - 1;
		private static int ENCRYPTEDDATA_START = 0;
		private static int ENCRYPTEDDATA_ENCRYPTED = 1;
		private static int ENCRYPTEDDATA_DECRYPTED = 2;
		private int stateOfObject = 0;
		private de.osci.osci12.encryption.EncryptedData encryptedDataObject = null;
		internal System.Collections.ArrayList roles;
		internal System.Collections.ArrayList readers;
		internal System.Collections.ArrayList attachments;
		private System.Collections.Hashtable encryptedKeyList;
		private MessagePart content = null;
		private OSCIMessage msg = null;
		private javax.crypto.SecretKey secretKey = null;

		private void  InitBlock() {
			roles = new System.Collections.ArrayList();
			readers = new System.Collections.ArrayList();
			attachments = new System.Collections.ArrayList();
			encryptedKeyList = new System.Collections.Hashtable();
		}

		/// <summary> Liefert die ReferenceId des EncryptedData Elements.
		/// </summary>
		/// <value>ReferenceID
		/// </value>
		virtual public System.String ReferenceId {
			get {
				return id;
			}	
		}
	
		/// <summary>Liefert den Identifier des symmetrischen Verschlsselungsalgorithmus.
		/// </summary>
		/// <value>Algorithmus
		/// </value>
		virtual public System.String SymEncryptionMethod {
			get {
				return encryptedDataObject.EncryptionMethodAlgorithm;
			}	
		}

		/// <summary> Liefert die eingestellten Attachment-Objekte des ContentContainer.
		/// </summary>
		/// <value> Array der zugehrigen Attachments
		/// </value>
		virtual public Attachment[] Attachments {
			get {
				return (Attachment[]) this.attachments.ToArray(typeof(Attachment));
			}	
		}

		/// <summary> Liefert die eingestellten Role-Objekte des EncryptedData-Objektes,
		/// welche fr die Signatur und/oder Verschlsselung verwendet wurden.
		/// </summary>
		/// <value> Array der verwendeten Role-Objekte.
		/// </value>
		virtual public Role[] Roles {
			get {
				return (Role[]) this.roles.ToArray(typeof(Role));
			}	
		}

		/// <summary> Liefert die eingestellten Role-Objekte des EncryptedData-Objektes,
		/// mit dem die Daten verschlsselt wurden.
		/// </summary>
		/// <value>Array der Verschlsselungs-Role-Objekte.
		/// </value>
		virtual public Role[] Readers {
			get {
				return (Role[]) this.roles.ToArray(typeof(Role));
			}			
		}
		virtual public string NS {
			get {
				return ns;
			}	
			set{
				ns=value;
			}
		}
		/**
		 * Interner Konstruktor fr das Hinzufgen von verschlsselten Attachments zur Nachricht
		 * @param attachment
		 * @throws IllegalArgumentException
		 * @throws IOException
		 */
		EncryptedDataOSCI(Attachment attachment)
		{
		InitBlock();
		typeOfData = TYPE_OF_DATA_ATTACHMENT;

		if (attachment == null)
		throw new System.ArgumentException(DialogHandler.text.getString(DialogHandler.text.getString("invalid_thirdargument") + " attachment = null"));

		typeOfData = TYPE_OF_DATA_ATTACHMENT;
		this.attachments.Add(attachment);

		if (!attachment.Encrypted)
		throw new System.ArgumentException(DialogHandler.text.getString(DialogHandler.text.getString("error_unencrypted_attachment")));

		this.secretKey = attachment.secretKey;

		if (log.DebugEnabled)
		log.debug("Secret-Key des Attachments wird verwendet.");

		de.osci.osci12.encryption.CipherReference ref_renamed = new de.osci.osci12.encryption.CipherReference("cid:" + attachment.RefID);
		RefID="Attachment" + attachment.RefID;
		encryptedDataObject = new de.osci.osci12.encryption.EncryptedData(ref_renamed, secretKey.GetAlgoString(),	RefID,secretKey);
		this.stateOfObject = ENCRYPTEDDATA_ENCRYPTED;

		KeyInfo keyInfo = new KeyInfo();
		keyInfo.MgmtData=Base64.encode(attachment.secretKey.Encoded());
		encryptedDataObject.KeyInfo=keyInfo;
		this.content = attachment;
	}

	/**
	 * Legt ein EncryptedData-Objekt fr AES-256-Verschlsselung an, welches
	 * als zu verschlsselnden Inhalt das bergebenen ContentContainer-Objekt
	 * enthlt. Es wird fr die symmetrische Verschlsselung ein
	 * TripleDES-Schlssel erzeugt.
	 *
	 * @param coco Inhaltsdatencontainer mit den zu verschlsselnden Daten
	 * @throws NoSuchAlgorithmException wenn ein nicht untersttzter Algorothmus
	 * bergeben wurde
	 */
	public EncryptedDataOSCI(ContentContainer coco):this(null, Constants.SYMMETRIC_CIPHER_ALGORITHM_AES256, coco)
	{;}
	public EncryptedDataOSCI(String symmetricCipherAlgorithm, ContentContainer coco):this(null, symmetricCipherAlgorithm, coco){;}

	/// <summary> Interner Konstruktor fr das Hinzufgen von verschlsselten Attachments zur Nachricht
		/// </summary>
		/// <param name="secretKey">geheimer Schlssel
		/// </param>
		/// <param name="attachment">attachment 
		/// </param>
		/// <exception cref="System.ArgumentException">
		/// </exception>
		internal EncryptedDataOSCI(javax.crypto.SecretKey secretKey, Attachment attachment) 
{
			InitBlock();
			typeOfData = TYPE_OF_DATA_ATTACHMENT;
			if (attachment == null) {
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_thirdargument") + " attachment is null");
			}
			typeOfData = TYPE_OF_DATA_ATTACHMENT;
			this.attachments.Add(attachment);
			if (!attachment.Encrypted) {
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_thirdargument") + " attachment must be encrypted");
			}
			this.secretKey = attachment.secretKey;
			log.debug("Secret-Key Des Attachment wird verwendet. Info: " +  new String(secretKey.Encoded().ToString().ToCharArray()));
		    de.osci.osci12.encryption.CipherReference ref_Renamed = new de.osci.osci12.encryption.CipherReference("cid:" + attachment.RefID);
			RefID="Attachment"+attachment.RefID;//pr mai
			encryptedDataObject = new de.osci.osci12.encryption.EncryptedData(ref_Renamed, secretKey.GetAlgoString(), RefID, secretKey);
			this.stateOfObject = ENCRYPTEDDATA_ENCRYPTED;
			KeyInfo keyInfo = new KeyInfo();
			keyInfo.MgmtData=Base64.encode(attachment.secretKey.Encoded());
			encryptedDataObject.KeyInfo = keyInfo;
			this.content = attachment;
		}
		
		/// <summary> Legt eine EncryptedData-Objekt an, welches als zu verschlsselnden Inhalt
		/// das bergebene MessagePart-Objekt enthlt.
		/// </summary>
		/// <param name="secretKey">geheimer Schlssel zum Verschlsseln der Daten, der
		/// wiederum in verschlsselter Form in die EncryptedKey-Elemente eingetragen
		/// wird. Wird als dritter Parameter ein Attachment-Objekt bergeben, so wird
		/// dieser erste Parameter ignoriert und der geheime Schlssel aus dem
		/// Attachment-Objekt bernommen.
		/// </param>
		/// <param name="encryptionMethodAlgorithm">Verschlsselungsalgorithmus, mu z.Zt.
		/// "http://www.w3.org/2001/04/xmlenc#tripledes-cbc" (TDES) sein.
		/// </param>
		/// <param name="coco">Inheltsdatenobjekt mit den zu verschlsselnden Daten vom
		/// Typ ContentContainer, EncryptedData oder Attachment
		/// </param>
		/// <exception cref="System.Exception">im Fehlerfall
		/// </exception>
		public EncryptedDataOSCI(javax.crypto.SecretKey secretKey, System.String encryptionMethodAlgorithm, ContentContainer coco) {
			InitBlock();
			transformers.Add(can);
			id = typ + (++idNr);
			if (encryptionMethodAlgorithm == null)encryptionMethodAlgorithm=Constants.SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC;
			if (secretKey == null) {
				secretKey = de.osci.osci12.encryption.Crypto.createSymKey(encryptionMethodAlgorithm);
			}
			encryptionMethodAlgorithm=secretKey.GetAlgoString();
			if (!encryptionMethodAlgorithm.Equals("http://www.w3.org/2001/04/xmlenc#tripledes-cbc")) {
				if (!encryptionMethodAlgorithm.StartsWith("http://www.w3.org/2001/04/xmlenc#aes")) {
					throw new System.Exception(DialogHandler.text.getString("encryption_algorithm_not_supported" + " encryptionMethodAlgorithm"));
				}
			}
			if (coco == null) {
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_thirdargument") + " coco should not be 'null'");
			}
			typeOfData = TYPE_OF_DATA_CONTENTCONTAINER;
			id = "encdata_" + idNr++;
			this.secretKey = secretKey;
			this.content = coco;
			de.osci.osci12.encryption.CipherValue value_Renamed = new de.osci.osci12.encryption.CipherValue(coco);
			encryptedDataObject = new de.osci.osci12.encryption.EncryptedData(value_Renamed, encryptionMethodAlgorithm, id, this.secretKey);
			Attachment[] atts = ((ContentContainer) content).Attachments;
			if ((atts != null) && (atts.Length > 0)) {
				for (int i = 0; i < atts.Length; i++) {
					this.attachments.Add(atts[i]);
				}
			}
			for (int i = 0; i < coco.roles.Count; i++) {
				roles.Add(coco.roles[i]);
			}
            encryptedDataObject.encNS = MoreSupport.NewString(coco.ns, Constants.CHAR_ENCODING);

		}		
		
		/// <summary> Dieser Konstruktor wird beim parsen aufgerufen
		/// </summary>
		/// <param name="encryptedData">encryptedData
		/// </param>
		internal EncryptedDataOSCI(de.osci.osci12.encryption.EncryptedData encryptedData, OSCIMessage osciMsg) {
			InitBlock();
			this.encryptedDataObject = encryptedData;
			stateOfObject = ENCRYPTEDDATA_ENCRYPTED;
			this.id = encryptedData.Id;
			msg = osciMsg;
			de.osci.osci12.encryption.EncryptedKey[] encKeys = encryptedData.KeyInfo.EncryptedKeys;
			for (int i = 0; i < encKeys.Length; i++) {
				System.String uri = ((de.osci.osci12.encryption.EncryptedKey) encKeys[i]).KeyInfo.RetrievalMethod.URI;
				if (uri.StartsWith("#")) uri = uri.Substring(1); 
				Role role = osciMsg.getRoleForRefID(uri);
				if (role == null) {
					throw new System.IO.IOException("Das Role Object mit der RefId " + uri + " konnte in der OSCIMessage nicht gefunden werden ");
				}
				roles.Add(role);
				readers.Add(role);
			}
		}
		
		/// <summary><p>Diese Methode entschlsselt das EncryptedDataOSCI Objekt mit dem bergebenem Role Objekt.</p>
		/// <p>Die Verschlsselung wird aufgehoben, die Entschlsselten Informationen geparst und ein
		/// ContentContainer Objekt aufgebau. Decryptet das EncryptedData Object und liefert ein entschlsselten 
		/// ContentContainer als Return-Wert. </p>
		/// </summary>
		/// <param name="reader">enthlt die Entschlsselungsinformationen. Das Role-Objekt muss auf
		/// jeden Fall ein Decrypter Objekt zur Aufhebung der Verschlsselung enthalten.
		/// </param>
		/// <returns> Liefert den decrypted ContentContainer als Antwort
		/// </returns>
		/// <exception cref="System.ArgumentException">
		/// </exception>
		/// <exception cref="System.SystemException">
		/// </exception>
		public virtual ContentContainer decrypt(Role reader) {
			if (reader == null) {
				throw new System.ArgumentException(DialogHandler.text.getString("wrong_role_cipher_obj"));
			}
			String rdId = null;

			for (int i = 0; i < roles.Count; i++) {
				if (reader.CipherCertificate.Equals(((Role) roles[i]).CipherCertificate)) {
					rdId = ((Role) roles[i]).CipherCertificateId;
					break;
				}
			}

			if (rdId == null)
				throw new dothelpers.IllegalArgumentException(DialogHandler.text.getString("no_encryption_for_role"));



			if (stateOfObject == ENCRYPTEDDATA_DECRYPTED) {
				throw new System.SystemException(DialogHandler.text.getString("invalid_stateofobject") + " Object is decrypted.");
			}
			log.debug("Start");
			if (rdId.IndexOf("_") >= 0) rdId = rdId.Substring(rdId.IndexOf('_'));
			de.osci.osci12.encryption.EncryptedKey encKey = this.encryptedDataObject.findEncrypedKey(rdId);

			log.debug("Anzahl der Empfnger: " + encryptedKeyList.Count);
			if ((encKey == null) || !reader.hasCipherPrivateKey()) {
				System.Collections.IEnumerator enum_Renamed = encryptedKeyList.Keys.GetEnumerator();
				while (enum_Renamed.MoveNext()) {
					log.debug("Verschlsselt fr: " + ((de.osci.osci12.roles.Role) enum_Renamed.Current).CipherCertificateId);
				}
				throw new System.ArgumentException(DialogHandler.text.getString("wrong_role_cipher_obj"));
			}
			log.debug("key: " + encKey.KeyInfo.RetrievalMethod.URI);
			// es wurde ein EncryptedKey gefunden
			// endschlsseln des symmetrischen Schlssels
			System.IO.Stream keyIn = encKey.CipherData.CipherValue.CipherValueBuffer.InputStream;
			keyIn.Seek(0,System.IO.SeekOrigin.Begin);

			Base64InputStream base64In = new Base64InputStream(keyIn);
			byte[] dt = de.osci.helper.Tools.readBytes(base64In);
			byte[] decryptedKey = reader.Decrypter.decrypt(dt);
			
			// Unterscheidung ob es sich um eine CipherRef oder Cipher Value handelt
			if (encryptedDataObject.CipherData.CipherReference != null) {
				de.osci.osci12.encryption.CipherReference cipherRef = encryptedDataObject.CipherData.CipherReference;
				System.String refId = encryptedDataObject.Id;
				if (refId.StartsWith("Attachment")) {
					log.error("EncryptedData object is an attachment.");
					throw new OSCICipherException(DialogHandler.text.getString("decryption_error") + " EncryptedData object is an attachment.");
				}
				else if (refId.StartsWith("encOSCI_Msg")) {
					log.debug("Es handelt sich um eine verschlsselte OSCI-Nachricht.");
					throw new OSCICipherException(DialogHandler.text.getString("decryption_error") + " EncryptedData object is a OSCI-Message.");
				}
				else {
					log.debug("Es handelt sich um eine CipherReference unbekannter herkunft.");
					throw new OSCICipherException(DialogHandler.text.getString("decryption_error") + " EncryptedData object is a unknown CipherReferenz.");
				}
			}
			else {
				log.debug("Es handelt sich um eine CipherValue.");
				System.IO.Stream in_Renamed = encryptedDataObject.CipherData.CipherValue.CipherValueBuffer.InputStream;
				de.osci.osci12.common.Base64InputStream bin = new de.osci.osci12.common.Base64InputStream(in_Renamed);
				SymCipherInputStream cin = new SymCipherInputStream(bin, de.osci.osci12.encryption.Crypto.createSymKey(decryptedKey,encryptedDataObject.EncryptionMethodAlgorithm), false);
				this.stateOfObject = ENCRYPTEDDATA_DECRYPTED;
				return parseInputStream(cin);
			}
		}
		
		/// <summary> Diese Methode erstellt die EncryptedData Strukturen ohne den symetrischen Schlssel nocheinmal
		/// zu verschlsseln. Dies kann bei vielen Verschlsselungen zu einer schnelleren Verarbeitung mit wenig
		/// PIN eingaben fhren.
		/// </summary>
		/// <param name="encryptedSymKey">verschlsselter symmetrischer Schlssel.
		/// </param>
		/// <param name="reader">Role-Objekt, welches den Leser reprsentiert.
		/// </param>
		public virtual void encrypt(byte[] encryptedSymKey, Role reader) {
			if (!readers.Contains(reader)) {
				roles.Add(reader);
				this.readers.Add(reader);
			}
			this.stateOfObject = ENCRYPTEDDATA_ENCRYPTED;
			log.debug("Encrypted-Data Methode encrypt mit :" + reader.CipherCertificateId); //+ " encryptedSymKey: " +
			KeyInfo keyInfo = null;
			if (content == null) {
				keyInfo = new KeyInfo("#" + reader.CipherCertificate);
			}
			else {
				keyInfo = new KeyInfo("#" + reader.CipherCertificateId);
			}
			de.osci.osci12.encryption.CipherValue cipherValue = new de.osci.osci12.encryption.CipherValue(encryptedSymKey);
			cipherValue.useSecretKey(false);
			de.osci.osci12.encryption.EncryptedKey encryptedKey = new de.osci.osci12.encryption.EncryptedKey(de.osci.osci12.common.Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5, cipherValue);
			log.debug("Encrypted Base64 CipherValue: " + new System.String(SupportClass.ToCharArray(cipherValue.CipherValueBytes)));
			encryptedKey.Id = "EncData_" + id + "_" + keyIDs;
			encryptedKey.KeyInfo = keyInfo;
			if (encryptedDataObject.KeyInfo == null) {
				encryptedDataObject.KeyInfo = new KeyInfo();
			}
			encryptedDataObject.KeyInfo.addEncryptedKey(encryptedKey);
			SupportClass.PutElement(encryptedKeyList, reader, encryptedKey);
			if (content is ContentContainer)createEncryptedAttachments((ContentContainer)content,encryptedSymKey, reader);
			log.debug("Fertig mit Encrypted-Data Methode encrypt mit.");
		}
		
		/// <summary> Diese Methode parst den soeben entschlsselten XML-Strom und liefert das Ergebnisobjekt zurck.
		/// </summary>
		/// <param name="in">Stream, der geparst wird.
		/// </param>
		/// <param name="reader">reader
		/// </param>
		/// <returns>  Erstelltes Object (derzeit nur coco)
		/// </returns>
		/// <exception cref="OSCICipherException">
		/// </exception>
		private ContentContainer parseInputStream(System.IO.Stream in_Renamed){
			ContentPackageBuilder copaBuilder = null;
			Canonizer can = new Canonizer(in_Renamed, null); 
			javax.xml.parsers.SAXParserFactory saxFactory = javax.xml.parsers.
			SAXParserFactory.newInstance();
			javax.xml.parsers.SAXParser parser = saxFactory.newSAXParser();
			XMLReader rd = parser.XMLReader;
			copaBuilder = new ContentPackageBuilder(rd, msg, can);
			rd.setContentHandler(copaBuilder);
			rd.parse(can);
			in_Renamed.Close();
			if (copaBuilder.LastCreatedObject is ContentContainer) {
				ContentContainer coco = (ContentContainer) copaBuilder.LastCreatedObject;
				decryptAttachments(coco);
				return coco;
			}
			else {
				throw new OSCICipherException("sax_exception");
			}
		}
		private void decryptAttachments(ContentContainer cc){
			EncryptedDataOSCI[] encDatas = cc.EncryptedData;

			for (int i = 0; i < encDatas.Length; i++) {
			if (encDatas[i].encryptedDataObject.CipherData.CipherReference != null) {
				String uri = encDatas[i].encryptedDataObject.CipherData.CipherReference.URI;
				if (uri.StartsWith("cid:")) {
					uri = uri.Substring(4);
				}
				encDatas[i].RefID=uri;

				System.Collections.Hashtable msgAtts = msg.attachments;
				Attachment att = (Attachment) msgAtts[uri];
				att.stateOfAttachment = Attachment.STATE_OF_ATTACHMENT_ENCRYPTED;
				att.encrypt = true;

				if (log.DebugEnabled){
					log.debug("das Attachment: " + att + " URI: " + uri);
				}
				att.secretKey = Crypto.createSymKey(Base64.decode
					(encDatas[i].encryptedDataObject.KeyInfo.MgmtData),encDatas[i].encryptedDataObject.EncryptionMethodAlgorithm);
				SupportClass.PutElement(cc.attachments,att.RefID, att);
				cc.removeEncryptedData(encDatas[i], false);
			}
			}

			Content[] cnts = cc.Contents;
			for (int i = 0; i < cnts.Length; i++) {
				if (cnts[i].ContentType == Content.CONTENT_CONTAINER) {
					decryptAttachments(cnts[i].ContentContainer);
				}
			}
		}
		
		/// <summary> Fgt beim Verschlsseln die encryptedData Elemente der Attachments hinzu.
		/// </summary>
		/// <param name="encryptedSymKey">encryptedSymKey
		/// </param>
		/// <param name="reader">reader
		/// </param>
		private void  createEncryptedAttachments(ContentContainer cc, byte[] encryptedSymKey, Role reader) {
				Attachment[] atts = cc.Attachments;
				if (atts != null) {
					ContentContainer coco = (ContentContainer) content;
					coco.stateOfObject = ContentContainer.STATE_OF_OBJECT_PARSING;
					for (int i = 0; i < atts.Length; i++) {
						log.debug("Ein weiteres Attachment.");
						EncryptedDataOSCI encData = new EncryptedDataOSCI(atts[i]);
						encData.setNSPrefixes(coco.soapNSPrefix, coco.osciNSPrefix, coco. dsNSPrefix, coco.xencNSPrefix, coco. xsiNSPrefix);
						encData.NS=MoreSupport.NewString(coco.ns, Constants.CHAR_ENCODING);
						coco.addEncryptedData(encData);
					}
					Role[] roles = ((ContentContainer) content).Roles;
					for (int i = 0; i < roles.Length; i++) {
						if (!this.roles.Contains(roles[i])) {
							this.roles.Add(roles[i]);
						}
					}
				}
				Content[] cnts = cc.Contents;
				for (int i = 0; i < cnts.Length; i++) {
					if (cnts[i].ContentType == Content.CONTENT_CONTAINER)
						createEncryptedAttachments(cnts[i].ContentContainer, encryptedSymKey, reader);
				}
//			}
		}
		
		/// <summary> Verschlsselt den geheimen Schlssel und fgt ihn als EncryptedKey-Element
		/// dem EncyptedData-Element hinzu. Auerdem wird das Rollen-Objekt der Parent-
		/// Nachricht (des MessagePart-Objektes) hinzugefgt, so dass das
		/// Verschlsselungszertifikat in der Nachricht enthalten ist.
		/// </summary>
		/// <param name="reader">Rollen-Objekt, fr welches verschlsselt werden soll.
		/// </param>
		/// <exception cref="OSCICipherException">wenn dem Rollen-Objekt das erforderliche
		/// Verschlsselungszertifikat fehlt.
		/// </exception>
		public virtual void  encrypt(de.osci.osci12.roles.Role reader) {
			if (!readers.Contains(reader)) {
				roles.Add(reader);
				readers.Add(reader);
			}
			this.stateOfObject = ENCRYPTEDDATA_ENCRYPTED;
			log.debug("Encrypted-Data Methode encrypt mit :" + reader.CipherCertificateId + " Es handelt sich um ein EncryptedData Objekt vom Typ:" + this.typeOfData);
			if (secretKey == null) {
				throw new OSCICipherException(DialogHandler.text.getString("encryption_error") + " no secretKey");
			}
			log.debug("Secret Key: " + new String(secretKey.Encoded().ToString().ToCharArray()));
			byte[] encryptedSymKey = de.osci.osci12.encryption.Crypto.doRSAEncryption(reader.CipherCertificate, secretKey.Encoded());
			encrypt(encryptedSymKey, reader);
		}
		
		/// <summary> Interne Methode, wird von Anwendungen normalerweise nicht aufgerufen.
		/// </summary>
		/// <param name="out_Renamed">out
		/// </param>
		/// <exception cref="System.IO.IOException">
		/// </exception>
		/// <exception cref="OSCICipherException">
		/// </exception>
		/// <exception cref="System.SystemException">
		/// </exception>
		public override void  writeXML(System.IO.Stream out_Renamed) {
			writeXML(out_Renamed, true);
		}
		internal void writeXML(System.IO.Stream out_Renamed, bool inner) {
			if (stateOfObject < ENCRYPTEDDATA_ENCRYPTED) {
				throw new System.SystemException(DialogHandler.text.getString("invalid_stateofobject") + " not encrypted.");
			}
			 encryptedDataObject.Id=RefID;
			encryptedDataObject.writeXML(out_Renamed,inner);
		}
		
		static EncryptedDataOSCI() {
			log = LogFactory.getLog(typeof(EncryptedDataOSCI));
		}
	}
}