namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;
	using System.Security.Cryptography.X509Certificates;

	/// <summary><p><H4>Annahmeauftrag</H4></p>
	/// Diese Klasse reprsentiert Nachrichtenobjekte fr Annahmeauftrge.
	/// Der Intermedir erzeugt nach dem Erhalt eines Weiterleitungsauftrags eine
	/// Instanz dieser Klasse und sendet die Nachricht an den Empfnger (hier als Supplier).
	/// Als Antwort auf diese Nachricht mu der Empfnger ein
	/// ResponseToAcceptDelivery-Nachrichtenobjekt mit einer Rckmeldung (Feedback)
	/// aufbauen und an den Intermedir zurcksenden.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ResponseToAcceptDelivery">
	/// </seealso>
	public class AcceptDelivery:OSCIRequest,ContentPackageI {
		
		public ProcessCardBundle ProcessCardBundle = null;
		
		/// <summary> Diese Methode liefert den Laufzettel der Zustellung zurck oder null,
		/// wenn bei der Verarbeitung der Nachricht ein Fehler aufgetreten ist.
		/// Die Informationen im Laufzettel knnen auch direkt ber die einzelnen
		/// getX()-Methoden ausgewertet werden.
		/// </summary>
		/// <value> Der Laufzettel als ProcessCardBundle-Objekt (im Fehlerfall null)
		/// </value>
		/// <seealso cref="TimestampCreation()">
		/// </seealso>
		/// <seealso cref="TimestampForwarding()">
		/// </seealso>
		/// <seealso cref="Inspections()">
		/// </seealso>
		/// <seealso cref="Subject()">
		/// </seealso>
		/// <seealso cref="RecentModification()">
		/// </seealso>
		/// <seealso cref="MessageId()">
		/// </seealso>
		virtual public ProcessCardBundle processCardBundle {
			get {
				return ProcessCardBundle;
			}	
		}
	
		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des Eingangs
		/// des Weiterleitungsauftrags beim Intermedir.
		/// </summary>
		/// <value> Der Zeitstempel der Einreichung beim Intermedir.
		/// </value>
		/// <seealso cref="processCardBundle()">
		/// </seealso>
		virtual public Timestamp TimestampCreation {
			get {
				return processCardBundle.Creation;
			}	
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des
		/// vollstndigen Aufbaus des Annahmeauftrags vom Intermedir fr den Empfnger.
		/// </summary>
		/// <value> Der Zeitstempel der Erstellung des Annahmeauftrags durch den Intermedir.
		/// </value>
		/// <seealso cref="processCardBundle()">
		/// </seealso>
		virtual public Timestamp TimestampForwarding {
			get {
				return processCardBundle.Forwarding;
			}	
		}

		/// <summary> Liefert die Ergebnisse der Zertifikatsprfungen in Form von Inspection-Objekten,
		/// die im ProcessCardBundle-Objekt enthalten sind.
		/// </summary>
		/// <value> Die Prfergebnisse.
		/// </value>
		/// <seealso cref="processCardBundle()">
		/// </seealso>
		virtual public Inspection[] Inspections {
			get {
				return processCardBundle.Inspections;
			}
		}

		/// <summary> Liefert den Betreff der Nachricht.
		/// </summary>
		/// <value> Der Betreff der Zustellung.
		/// </value>
		/// <seealso cref="processCardBundle()">
		/// </seealso>
		virtual public System.String Subject {
			get {
				return processCardBundle.subject;
			}	
		}

		/// <summary> Liefert das Datum der letzten nderung des Laufzettels. Das Format
		/// entspricht dem XML-Schema nach http://www.w3.org/TR/xmlschema-2/#dateTime
		/// </summary>
		/// <value> Datum der letzten nderung.
		/// </value>
		/// <seealso cref="processCardBundle()">
		/// </seealso>
		virtual public System.String RecentModification {
			get {
				return processCardBundle.recentModification;
			}	
		}

		/// <summary> Liefert die in die Nachricht eingestellten (unverschlsselten) Inhaltsdaten als ContentContainer-Objekte.
		/// </summary>
		/// <value> Enthaltene ContentContainer mit Inhaltsdaten.
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>
		override public ContentContainer[] ContentContainer {
			get {
				return base.ContentContainer;
			}	
		}

		/// <summary> Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
		/// als EncryptedData-Objekte.
		/// </summary>
		/// <value> Enthaltenes EncryptedData-Objekt mit verschlsselten Inhaltsdaten.
		/// </value>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		override public EncryptedDataOSCI[] EncryptedData {
			get {
				return base.EncryptedData;
			}	
		}

		/// <summary> Liefert die Message-ID der Nachricht.
		/// </summary>
		/// <value> Die Message-ID.
		/// </value>
		virtual public System.String MessageId {
			get {
				return processCardBundle.MessageId;
			}	
		}

		/// <summary> Diese Methode registriert (statisch) eine OSCIDataSource-Implementierung,
		/// die fr die Speicherung aller eingehenden Nachrichten dieses Typs
		/// verwendet wird. Beim Empfang einer Nachricht wird vom registrierten
		/// OSCIDataSource-Objekt eine neue Instanz geholt (OSCIDataSource.newInstance())
		/// und die Nachricht (der eingehende Bytestrom) in deren OutputStream geschrieben.
		/// </summary>
		/// <value>inputDataSourceImpl
		/// </value>
		/// <seealso cref="de.osci.osci12.extinterfaces.OSCIDataSource">
		/// </seealso>
		public static OSCIDataSource InputDataSourceImpl {
			set {
				inputDataSourceImpl = value;
			}	
		}
		
		/// <summary> Liefert die Instanz des registrierten OSCIDataSource-Objektes, welches
		/// fr die Speicherung der Nachricht beim Empfang verwendet wurde.
		/// Die Methode liefert null, wenn keine OSCIDataSource-Implementierung
		/// registriert wurde.
		/// </summary>
		/// <value> Instanz von OSCIDataSource.
		/// </value>
		virtual public OSCIDataSource InputDataSource {
			get {
				return inputDataSource;
			}
		}

		new private static Log log;
		private static OSCIDataSource inputDataSourceImpl;
		private OSCIDataSource inputDataSource;
		public DialogHandler fwdDH;
		
		/// <summary> Konstruktor fr den Supplier Parser
		/// </summary>
		/// <param name="dh">dh
		/// </param>
		internal AcceptDelivery(DialogHandler dh):base(dh) {
			messageType = ACCEPT_DELIVERY;
			addressee = ((Addressee) dh.Supplier);
		}
		
		internal AcceptDelivery() {
			messageType = ACCEPT_DELIVERY;
		}
		
		/// <summary> Konstruktor fr den Intermedir
		/// </summary>
		/// <param name="fd">ForwardDelivery
		/// </param>
		internal AcceptDelivery(ForwardDelivery fd):base(new DialogHandler((Intermed) fd.dialogHandler.Supplier, fd.addressee, (de.osci.osci12.extinterfaces.TransportI) null)) {
			messageType = ACCEPT_DELIVERY;
			uriReceiver = fd.uriReceiver;
			addressee = fd.addressee;
			originator = fd.originator;
			desiredLanguagesH = new DesiredLanguagesH(fd.desiredLanguagesH.LanguageList);

			ContentContainer[] con = fd.ContentContainer;
			for (int i = 0; i < con.Length; i++) {
				addContentContainer(con[i]);
			}

			EncryptedDataOSCI[] enc = fd.EncryptedData;
			for (int i = 0; i < enc.Length; i++) {
				addEncryptedData(enc[i]);
			}
			
			Attachment[] att = fd.Attachments;
			for (int i = 0; i < att.Length; i++) {
				addAttachment(att[i]);
			}
			for (int i = 0; i < fd.OtherAuthors.Length; i++)
				otherAutors[fd.OtherAuthors[i].id]=fd.OtherAuthors[i];
			for (int i = 0; i < fd.OtherReaders.Length; i++)
				otherReaders[fd.OtherReaders[i].id]=fd.OtherReaders[i];

			dialogHandler.Encryption=fd.DialogHandler.Encryption;
			dialogHandler.Controlblock.Response = null;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			dialogHandler.Controlblock.ConversationID = null;
			dialogHandler.Controlblock.SequenceNumber = - 1;
			fwdDH = fd.DialogHandler;
		}
		
		/// <summary> Versendet die Nachricht und liefert die Antwortnachricht zurck.
		/// Diese Methode wirft eine Exception, wenn beim Aufbau oder Versand der
		/// Nachricht ein Fehler auftritt. Fehlermeldungen vom Intermedir mssen
		/// dem Feedback der Antwortnachricht entnommen werden.
		/// </summary>
		/// <returns> das Antwortnachricht-Objekt
		/// </returns>
		// <-- Storestreams:
		internal virtual ResponseToAcceptDelivery send() {
			return (ResponseToAcceptDelivery) transmit(null, null);
		}
		// <-- Storestreams:
		
		/// <summary> Entfernt einen Inhaltsdatencontainer aus der Nachricht.
		/// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
		public override void  removeContentContainer(ContentContainer container) {
			base.removeContentContainer(container);
		}
				
		/// <summary> Fgt der Nachricht einen EncryptedData-Eintrag mit verschlsselten
		/// Inhaltsdaten hinzu.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="EncryptedDataOSCI"> 
		/// </seealso>
		public override void  addEncryptedData(EncryptedDataOSCI encData) {
			base.addEncryptedData(encData);
		}
		
		/// <summary> Entfernt einen EncryptedData-Eintrag mit verschlsselten Daten
		/// aus der Nachricht.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="addEncryptedData">
		/// </seealso>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public override void  removeEncryptedData(EncryptedDataOSCI encData) {
			base.removeEncryptedData(encData);
		}
		
		//		void setMessageId(String messageId) {
		//			this.messageId = messageId;
		//		}
		
		public override void  compose() 
		{
			base.compose();
			if (processCardBundle == null) 
			{
				throw new System.SystemException("Kein Laufzettel eingestellt.");
			}
			System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
			processCardBundle.writeXML(out_Renamed);

			char[] tmpChar;
			byte[] tmpByte;
			tmpByte = out_Renamed.GetBuffer();
			tmpChar = new char[out_Renamed.Length];
			System.Array.Copy(tmpByte, 0, tmpChar, 0, tmpChar.Length);
			osciH = new OsciH("acceptDelivery", new System.String(tmpChar));
			 if (dialogHandler.Client.hasCipherCertificate())
			 {
				if (intermediaryCertificatesH == null) {
					intermediaryCertificatesH = new IntermediaryCertificatesH();
				}
				intermediaryCertificatesH.CipherCertificateIntermediary = (Intermed) dialogHandler.Client;
			}
			createNonIntermediaryCertificatesH();
			
			if (body == null) {
				body = new Body(ContentContainer, EncryptedData);
			}
			stateOfMsg |= STATE_COMPOSED;
		}

		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);	
			}

			// DesiredLanguage
			desiredLanguagesH.writeXML(out_Renamed);
			osciH.writeXML(out_Renamed);
			
			if (intermediaryCertificatesH != null) {
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		
		static AcceptDelivery() {
			log = LogFactory.getLog(typeof(AcceptDelivery));
		}
	}
}