namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon; // Storestreams
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Zustellungsabholauftrag</H4></p>
	/// Mit dieser Klasse werden Nachrichtenobjekte fr Zustellungsabholauftrge
	/// angelegt. Clients knnen hiermit maximal eine Nachricht vom Intermedir
	/// abrufen.  Als Antwort auf diese Nachricht erhalten sie vom Intermedir
	/// ein ResponseToFetchDelivery-Nachrichtenobjekt, welches eine Rckmeldung ber den
	/// Erfolg der Operation (getFeedback()) und ggf. die gewnschte Nachricht enthlt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p>
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ResponseToFetchDelivery">
	/// </seealso>
	public class FetchDelivery:OSCIRequest {

		new private static Log log;
		internal System.String selectionRule;
		private int selectionMode = - 1;
			
		/// <summary> Ruft die Auswahlregel fr die abzuholende Nachricht ab, oder legt diese fest. Der Inhalt des
		/// bergebenen Strings hngt vom gewhlten Auswahlmodus ab und kann
		/// entweder in einer Base64-codierten Message-Id oder einem Datum bestehen.
		/// Das Format eines Datums mu dem XML-Schema nach
		/// <a href="http://www.w3.org/TR/xmlschema-2/#dateTime">
		/// http://www.w3.org/TR/xmlschema-2/#dateTime</a> (ISO 8601-Format) entsprechen.
		/// </summary>
		/// <value>Message-Id oder Datum
		/// </value>
		/// <seealso cref="SelectionMode()">
		/// </seealso>
		virtual public System.String SelectionRule {
			get {
				return selectionRule;
			}	
			set {
				this.selectionRule = value;
			}
		}
	
		/// <summary> Ruft den Auswahlmodus ab, oder setzt diesen. Mgliche Werte sind
		/// SELECT_BY_MESSAGE_ID, SELECT_BY_DATE_OF_RECEPTION oder NO_SELECTION_RULE
		/// (default).
		/// </summary>
		/// <value>Auswahlmodus
		/// </value>
		/// <seealso cref="SelectionRule()">
		/// </seealso>
		virtual public int SelectionMode {
			get {
				return selectionMode;
			}	
			set {
				if ((value != SELECT_BY_MESSAGE_ID) && 
					(value != SELECT_BY_DATE_OF_RECEPTION) && 
					(value != NO_SELECTION_RULE)) {
					throw new System.ArgumentException("Ungltiger Auswahlmodus. " + "Es sind nur die Werte SELECT_BY_MESSAGE_ID, SELECT_BY_DATE_OF_RECEPTION und NO_SELECTION_RULE erlaubt.");
				}
					
				this.selectionMode = value;
			}
		}

		internal FetchDelivery() {
			messageType = FETCH_DELIVERY;
		}
			
		/// <summary> Legt ein Nachrichtenobjekt fr einen Zustellungsabholauftrag an.
		/// </summary>
		/// <param name="dh">DialogHandler-Objekt des Dialogs, innerhalb dessen die Nachricht
		/// versendet werden soll.
		/// </param>
		/// <seealso cref="de.osci.osci12.common.DialogHandler">
		/// </seealso>
		public FetchDelivery(DialogHandler dh):base(dh) {
			messageType = FETCH_DELIVERY;
			originator = ((Originator) dh.Client);
				
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			dialogHandler.Controlblock.SequenceNumber = dialogHandler.Controlblock.SequenceNumber + 1;
		}
			
		/// <summary> Versendet die Nachricht und liefert die Antwortnachricht zurck.
		/// Diese Methode wirft eine Exception, wenn beim Aufbau oder Versand der
		/// Nachricht ein Fehler auftritt. Fehlermeldungen vom Intermedir mssen
		/// dem Feedback der Antwortnachricht entnommen werden.
		/// </summary>
		/// <returns> das Antwortnachricht-Objekt
		/// </returns>
		// --> Storestreams:
		public virtual ResponseToFetchDelivery send() {
			return (ResponseToFetchDelivery) transmit(null, null);
		}

		public virtual ResponseToFetchDelivery send(System.IO.Stream storeOutput, System.IO.Stream storeInput) {
			return (ResponseToFetchDelivery) transmit(storeOutput, storeInput);
		}
		// <-- Storestreams:

		public override void  compose() 
		{
			base.compose();
			if (dialogHandler.Controlblock.Challenge == null) 
			{
				throw new System.SystemException("Kein Challenge-Wert in DialogHandler eingestellt.");
			}
				
			if (dialogHandler.Controlblock.Response == null) {
				throw new System.SystemException("Kein Response-Wert in DialogHandler eingestellt.");
			}
				
			if (dialogHandler.Controlblock.ConversationID == null) {
				throw new System.SystemException("Keine Conversation-ID in DialogHandler eingestellt.");
			}
				
			if (dialogHandler.Controlblock.SequenceNumber == - 1) {
				throw new System.SystemException("Kein Squenznummer in DialogHandler eingestellt.");
			}
			System.Text.StringBuilder selection = new System.Text.StringBuilder("<"); 
			selection.Append(osciNSPrefix); 
			selection.Append(":SelectionRule><"); 
			selection.Append(osciNSPrefix); 
				
			if (selectionMode == SELECT_BY_MESSAGE_ID) 
			{
				selection.Append(":MessageId>");
				selection.Append(de.osci.helper.Base64.encode(MoreSupport.ToByteArray(selectionRule,Constants.CHAR_ENCODING)));
				selection.Append("</"); 
				selection.Append(osciNSPrefix); 
				selection.Append(":MessageId></"); 
			} 
			else if (selectionMode == SELECT_BY_DATE_OF_RECEPTION) 
			{
				selection.Append(":ReceptionOfDelivery>");
				selection.Append(selectionRule);
				selection.Append("</"); 
				selection.Append(osciNSPrefix); 
				selection.Append(":ReceptionOfDelivery></"); 
			}
			else 
				selection.Remove(0, selection.Length); 
    
			if (selection.Length > 0) 
			{ 
				selection.Append(osciNSPrefix); 
				selection.Append(":SelectionRule>"); 
			}      
			osciH = new OsciH("fetchDelivery", selection.ToString());
			body = new Body("");
				
			stateOfMsg |= STATE_COMPOSED;
		}
			
		public override void  writeXML(System.IO.Stream out_Renamed) {
			log.debug("CreateMsg child");
			base.writeXML(out_Renamed);
				
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
			}
			// DesiredLanguage
			desiredLanguagesH.writeXML(out_Renamed);
			osciH.writeXML(out_Renamed);
				
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
			
		static FetchDelivery() {
			log = LogFactory.getLog(typeof(FetchDelivery));
		}
	}
}