namespace de.osci.osci12.messagetypes {
 
	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Dialoginitialisierungs-Nachrichtenobjekt</H4></p>
	/// Diese Klasse dient der Initialisierung eines expliziten Dialogs. Clients
	/// erhalten als Antwort auf diese Nachricht vom Intermedir ein
	/// Nachrichtenobjekt, welches in seinem ControlBlock die angeforderte
	/// ConversationId enthlt. Diese Id wird beim Empfang der Antwort an das
	/// verwendete DialogHandler-Objekt bergeben. Der Client mu lediglich dieses
	/// DialogHandler-Objekt fr alle weiteren Nachrichten verwenden, die innerhalb
	/// dieses Dialogs behandelt werden sollen.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p>
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.common.DialogHandler">
	/// </seealso>
	
	public class InitDialog:OSCIRequest {

		new private static Log log;
		
		/// <summary> Legt ein Nachrichtenobjekt zur Dialoginitialisierung an.
		/// </summary>
		/// <param name="dh">DialogHandler-Objekt, welches fr die
		/// folgenden Nachrichten initialisiert werden soll.
		/// </param>
		/// <exception cref="OSCIRoleException">wenn dem im bergebenen DialogHandler
		/// eingestellten Originator-Objekt das Verschlsselungszertifikat
		/// fehlt.
		/// </exception>
		/// <seealso cref="de.osci.osci12.common.DialogHandler">
		/// </seealso>		
		public InitDialog(DialogHandler dh):base(dh) {
			dh.Controlblock.Challenge = Tools.createRandom(10);
			dh.Controlblock.ConversationID = null;
			dh.Controlblock.SequenceNumber = - 1;
			dh.Controlblock.Response = null;
			messageType = INIT_DIALOG;
			originator = ((Originator) dh.Client);
		}
		
		/// <summary> Dieser Konstruktor wird nur fr die parser bentigt
		/// </summary>
		internal InitDialog() {
			messageType = INIT_DIALOG;
		}
		
		/// <summary> Versendet die Nachricht und liefert die Antwortnachricht zurck.
		/// Diese Methode wirft eine Exception, wenn beim Aufbau oder Versand der
		/// Nachricht ein Fehler auftritt. Fehlermeldungen vom Intermedir mssen
		/// dem Feedback der Antwortnachricht entnommen werden.
		/// </summary>
		/// <returns> das Antwortnachricht-Objekt
		/// </returns>
		// --> Storestreams:
		public virtual ResponseToInitDialog send() {
			return (ResponseToInitDialog) transmit(null, null);
		}
		// <-- Storestreams:
				
		public override void  compose() 
		{
			base.compose();
			if (nonIntermediaryCertificatesH == null) 
			{
				nonIntermediaryCertificatesH = new NonIntermediaryCertificatesH();
			}
			nonIntermediaryCertificatesH.CipherCertificateOriginator = (Originator) dialogHandler.Client;
			
			body = new Body("<" +  osciNSPrefix + ":initDialog></" +  osciNSPrefix + ":initDialog>");
			// Zur Sicherheit nochmal aufrumen
			if (dialogHandler.Controlblock.Challenge == null) {
				throw new System.SystemException("Kein Challenge-Wert in DialogHandler eingestellt.");
			}
			
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
			}
			
			// DesiredLanguage
			desiredLanguagesH.writeXML(out_Renamed);
			nonIntermediaryCertificatesH.writeXML(out_Renamed);
			
			completeMessage(out_Renamed);
		}
			
		static InitDialog() {
			log = LogFactory.getLog(typeof(InitDialog));
		}
	}
}