namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Zustellungsabholantwort-Nachrichtenobjekt</H4></p>
	/// Diese Klasse reprsentiert die Antwort des Intermedirs auf einen
	/// Zustellungsabholauftrag.
	/// Clients erhalten vom Intermedir eine Instanz dieser Klasse, die eine Rckmeldung
	/// ber den Erfolg der Operation (getFeedback()) sowie ggf. die angeforderten verschlsselten
	/// und/oder unverschlsselten Inhaltsdaten einschl. des zugehrigen Laufzettels enthlt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.FetchDelivery">
	/// </seealso>
	
	public class ResponseToFetchDelivery:OSCIResponseTo,ContentPackageI {
		/// <summary> Liefert die in die Nachricht eingestellten Attachment als Attachment-Objekte.
		/// </summary>
		/// <value> enthaltene Attachment mit Inhaltsdaten.
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.Attachment">
		/// </seealso>
		override public Attachment[] Attachments {
			get {
				return base.Attachments;
			}
		}

		/// <summary> Liefert die in die Nachricht eingestellten (unverschlsselten) Inhaltsdaten als ContentContainer-Objekte.
		/// </summary>
		/// <value> enthaltene ContentContainer mit Inhaltsdaten.
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>		
		override public ContentContainer[] ContentContainer {
			get {
				return base.ContentContainer;
			}
		}

		/// <summary> Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
		/// als EncryptedData-Objekte.
		/// </summary>
		/// <value> enthaltene EncryptedData-Objekt mit verschlsselten Inhaltsdaten.
		/// </value>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		override public EncryptedDataOSCI[] EncryptedData {			
			get {
				return base.EncryptedData;
			}			
		}

		/// <summary> Liefert die gesetzte Auswahlregel. Der Inhalt des zruckgegebenen
		/// Strings hngt vom gesetzten Auswahlmodus ab und kann
		/// entweder in einer Base64-codierten Message-Id oder einem Datum bestehen.
		/// Das Format eines Datums entspricht  dem XML-Schema nach
		/// <a href="http://www.w3.org/TR/xmlschema-2/#dateTime">
		/// http://www.w3.org/TR/xmlschema-2/#dateTime</a>.
		/// Wurde keine Regel gesetzt, wird als default null zurckgegeben.
		/// </summary>
		/// <value> Auswahlregel (Message-Id oder Datum)
		/// </value>
		/// <seealso cref="SelectionRule()">
		/// </seealso>
		// <getset></getset>
		virtual public System.String SelectionRule {
			get {
				return selectionRule;
			}	
		}

		internal System.String selectionRule;
		
		internal int SelectionMode = - 1;

		/// <summary> Liefert den gesetzten Auswahlmodus.
		/// </summary>
		/// <value> Auswahlmodus SELECT_BY_MESSAGE_ID, SELECT_BY_DATE_OF_RECEPTION
		/// oder NO_SELECTION_RULE
		/// </value>
		virtual public int selectionMode {
			get {
				return SelectionMode;
			}	
		}

		/// <summary> Diese Methode liefert den Laufzettel der Zustellung zurck oder null,
		/// wenn bei der Verarbeitung der Nachricht ein Fehler aufgetereten ist.
		/// Die Informationen im Laufzettel knnen auch direkt ber die einzelnen
		/// getX()-Methoden ausgewertet werden.
		/// </summary>
		/// <value> Laufzettel als ProcessCardBundle-Objekt, im Fehlerfall null
		/// </value>
		/// <seealso cref="TimestampCreation()">
		/// </seealso>
		/// <seealso cref="TimestampForwarding()">
		/// </seealso>
		/// <seealso cref="Inspections()">
		/// </seealso>
		/// <seealso cref="Subject()">
		/// </seealso>
		/// <seealso cref="RecentModification()">
		/// </seealso>
		/// <seealso cref="MessageId()">
		/// </seealso>
		virtual public ProcessCardBundle ProcessCardBundle {
			get {
				return processCardBundle;
			}
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des Eingangs
		/// des Zustellungsauftrags beim Intermedir.
		/// </summary>
		/// <value> Zeitstempel der Einreichung beim Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public Timestamp TimestampCreation {
			get {
				if (processCardBundle == null) {
					return null;
				}
				return processCardBundle.Creation;
			}	
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des
		/// vollstndigen Aufbaus der Abholantwort vom Intermedir fr den Empfnger.
		/// </summary>
		/// <value> Zeitstempel der Erstellung des Abholantwort durch den Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public Timestamp TimestampForwarding {
			get {
				if (processCardBundle == null) {
					return null;
				}
				return processCardBundle.Forwarding;
			}
		}

		/// <summary> Liefert die Ergebnisse der Zertifikatsprfungen in Form von Inspection-Objekten,
		/// die im ProcessCardBundle-Objekt enthalten sind.
		/// </summary>
		/// <value> Prfergebnisse
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>		
		virtual public Inspection[] Inspections {
			get {
				if (processCardBundle == null) {
					return null;
				}
				return processCardBundle.Inspections;
			}			
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <value> Betreff der Zustellung
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public System.String Subject {
			get {
				if (processCardBundle == null) {
					return null;
				}
				return processCardBundle.subject;
			}	
		}

		/// <summary> Liefert das Datum der letzten nderung des Laufzettels. Das Format
		/// entspricht dem XML-Schema nach http://www.w3.org/TR/xmlschema-2/#dateTime
		/// </summary>
		/// <value> Datum der letzten nderung.
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>		
		virtual public System.String RecentModification {
			get {
				if (processCardBundle == null) {
					return null;
				}
				return processCardBundle.recentModification;
			}
		}

		/// <summary> Liefert die Message-ID der Nachricht.
		/// </summary>
		/// <value> Message-ID
		/// </value>		
		virtual public System.String MessageId {
			get {
				if (processCardBundle == null) {
					return null;
				}
				return processCardBundle.MessageId;
			}
		}

		/// <summary> Diese Methode registriert (statisch) eine OSCIDataSource-Implementierung,
		/// die fr die Speicherung aller eingehenden Nachrichten dieses Typs
		/// verwendet wird. Beim Empfang einer Nachricht wird vom registrierten
		/// OSCIDataSource-Objekt eine neue Instanz geholt (OSCIDataSource.newInstance())
		/// und die Nachricht (der eingehende Bytestrom) in deren OutputStream geschrieben.
		/// </summary>
		/// <seealso cref="de.osci.osci12.extinterfaces.OSCIDataSource">
		/// </seealso>		
		public static OSCIDataSource InputDataSourceImpl {
			set {
				inputDataSourceImpl = value;
			}		
		}

		/// <summary> Liefert die Instanz des registrierten OSCIDataSource-Objektes, welches
		/// fr die Speicherung der Nachricht beim Empfang verwendet wurde.
		/// Die Methode liefert null, wenn keine OSCIDataSource-Implementierung
		/// registriert wurde.
		/// </summary>
		/// <value> Instanz von OSCIDataSource
		/// </value>
		virtual public OSCIDataSource InputDataSource {
			get {
				return inputDataSource;
			}	
		}

		new private static Log log;
		public ProcessCardBundle processCardBundle;
		
		private static OSCIDataSource inputDataSourceImpl;
		private OSCIDataSource inputDataSource;
		
		internal ResponseToFetchDelivery(FetchDelivery fetchDel, StoreDelivery storeDel):base(fetchDel.dialogHandler) {

			//@todo: hier werden die Rollenobjekte getauscht...
			addressee = storeDel.addressee;
			originator = storeDel.originator;
			messageType = RESPONSE_TO_FETCH_DELIVERY;
			
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			
			SelectionMode = fetchDel.SelectionMode;
			selectionRule = fetchDel.SelectionRule;
			
			if (storeDel == null) {
				setFeedback(new System.String[]{"9803"});
			}
			else {
				ContentContainer[] con = storeDel.ContentContainer;
				for (int i = 0; i < con.Length; i++) {
					addContentContainer(con[i]);
				}
				EncryptedDataOSCI[] enc = storeDel.EncryptedData;
				for (int i = 0; i < enc.Length; i++) {
					addEncryptedData(enc[i]);
				}
				Attachment[] att = storeDel.Attachments;
				for (int i = 0; i < att.Length; i++) {
					addAttachment(att[i]);
				}
				for (int i = 0; i < storeDel.OtherAuthors.Length; i++) otherAutors[storeDel.OtherAuthors[i].id]=storeDel.OtherAuthors[i];
				for (int i = 0; i < storeDel.OtherReaders.Length; i++) otherReaders[storeDel.OtherReaders[i].id]=storeDel.OtherReaders[i];
			}
		}
		
		internal ResponseToFetchDelivery(DialogHandler dh):base(dh) {
			originator = ((Originator) dh.Client);
			messageType = RESPONSE_TO_FETCH_DELIVERY;
		}
		
		public override void  compose() 
		{
			base.compose();
			createNonIntermediaryCertificatesH();			
			System.String selection = "";
			
			if (selectionMode == SELECT_BY_MESSAGE_ID) {
				selection = "<" + osciNSPrefix + ":fetchDelivery><" + osciNSPrefix + ":SelectionRule><" + osciNSPrefix + ":MessageId>" +
					de.osci.helper.Base64.encode(selectionRule)
					+ "</" + osciNSPrefix + ":MessageId></" + osciNSPrefix + ":SelectionRule></" + osciNSPrefix + ":fetchDelivery>";
			}
			else if (selectionMode == SELECT_BY_DATE_OF_RECEPTION) {
				selection = "<" + osciNSPrefix + ":fetchDelivery><" + osciNSPrefix + ":SelectionRule><" + osciNSPrefix + ":ReceptionOfDelivery>" 
					+ selectionRule + 
					"</" + osciNSPrefix + ":ReceptionOfDelivery></" + osciNSPrefix + ":SelectionRule></" + osciNSPrefix + ":fetchDelivery>";
			}
			else {
				selection = "<" + osciNSPrefix + ":fetchDelivery></" + osciNSPrefix + ":fetchDelivery>";
			}
			if (processCardBundle == null) {
				osciH = new OsciH("responseToFetchDelivery", writeFeedBack() + selection);
			}
			else {
				System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
				processCardBundle.writeXML(out_Renamed);
				
				char[] tmpChar;
				byte[] tmpByte;
				tmpByte = out_Renamed.GetBuffer();
				tmpChar = new char[out_Renamed.Length];
				System.Array.Copy(tmpByte, 0, tmpChar, 0, tmpChar.Length);
				osciH = new OsciH("responseToFetchDelivery", writeFeedBack() + selection + new System.String(tmpChar));
			}
			
			body = new Body(ContentContainer, EncryptedData);
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
			}
			osciH.writeXML(out_Renamed);
			
			if (intermediaryCertificatesH != null) {
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		
		static ResponseToFetchDelivery() {
			log = LogFactory.getLog(typeof(ResponseToFetchDelivery));
		}
	}
}