namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Abwicklungsantwort-Nachrichtenobjekt</H4></p>
	/// Diese Klasse reprsentiert die Antwort des Intermedirs auf einen
	/// Abwicklungsauftrag.
	/// Clients erhalten vom Intermedir eine Instanz dieser Klasse, die eine Rckmeldung
	/// ber den Erfolg der Operation (getFeedback()) sowie ggf. den zum Auftrag
	/// (Abwicklungs-/Bearbeitungsauftrag) gehrenden Laufzettel, den zur Antwort
	/// (Bearbeitungs-/Abwicklungsantwort) gehrenden Laufzettel und
	/// verschlsselte bzw. unverschlsselte Inhaltsdaten enthlt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.MediateDelivery">
	/// </seealso>
	public class ResponseToMediateDelivery:OSCIResponseTo, ContentPackageI
	{

		/// <summary> Diese Methode liefert den Laufzettel des Auftrags zurck oder null,
		/// wenn bei der Verarbeitung der Nachricht ein Fehler aufgetereten ist.
		/// Die Informationen im Laufzettel knnen auch direkt ber die einzelnen
		/// getX()-Methoden ausgewertet werden.
		/// </summary>
		/// <value> Laufzettel des Auftrags als ProcessCardBundle-Objekt, im Fehlerfall null
		/// </value>
		/// <seealso cref="ProcessCardBundleReply()">
		/// </seealso>
		/// <seealso cref="TimestampCreationRequest()">
		/// </seealso>
		/// <seealso cref="TimestampForwardingRequest()">
		/// </seealso>
		/// <seealso cref="InspectionsRequest()">
		/// </seealso>
		/// <seealso cref="SubjectRequest()">
		/// </seealso>
		/// <seealso cref="RecentModificationRequest()">
		/// </seealso>
		virtual public ProcessCardBundle ProcessCardBundleRequest {
			get {
				return processCardBundleRequest;
			}
			
		}

		/// <summary> Diese Methode liefert den Laufzettel der Antwort zurck oder null,
		/// wenn bei der Verarbeitung der Nachricht ein Fehler aufgetereten ist.
		/// Die Informationen im Laufzettel knnen auch direkt ber die einzelnen
		/// getX()-Methoden ausgewertet werden.
		/// </summary>
		/// <value> Laufzettel der Antwort als ProcessCardBundle-Objekt, im Fehlerfall null
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="TimestampCreationReply()">
		/// </seealso>
		/// <seealso cref="TimestampForwardingReply()">
		/// </seealso>
		/// <seealso cref="InspectionsReply()">
		/// </seealso>
		/// <seealso cref="SubjectReply()">
		/// </seealso>
		/// <seealso cref="RecentModificationReply()">
		/// </seealso>
		/// <seealso cref="MessageId()">
		/// </seealso>
		/// <seealso cref="RecentModificationReply()">
		/// </seealso>
		virtual public ProcessCardBundle ProcessCardBundleReply {
			get {
				return processCardBundleReply;
			}
		}

		/// <summary> Liefert den im Auftragslaufzettel enthaltenen Zeitstempel vom Zeitpunkt des Eingangs
		/// des Abwicklungsauftrags beim Intermedir.
		/// </summary>
		/// <value> Zeitstempel der Einreichung beim Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundleReply()">
		/// </seealso>
		/// <seealso cref="TimestampCreationReply()">
		/// </seealso>
		virtual public Timestamp TimestampCreationRequest {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleRequest.Creation;
			}
		}

		/// <summary> Liefert den im Antwortlaufzettel enthaltenen Zeitstempel vom Zeitpunkt des Eingangs
		/// der Bearbeitungsantwort beim Intermedir.
		/// </summary>
		/// <value> Zeitstempel der Einreichung beim Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundleReply()">
		/// </seealso>
		/// <seealso cref="TimestampCreationRequest()">
		/// </seealso>
		virtual public Timestamp TimestampCreationReply {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleReply.Creation;
			}
			
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des
		/// vollstndigen Aufbaus des Bearbeitungsauftrags vom Intermedir fr den Empfnger.
		/// </summary>
		/// <value> Zeitstempel der Erstellung des Bearbeitungsauftrags durch den Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="TimestampForwardingReply()">
		/// </seealso>
		virtual public Timestamp TimestampForwardingRequest {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleRequest.Forwarding;
			}
			
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des
		/// vollstndigen Aufbaus der Abwicklungsantwort vom Intermedir fr den Sender.
		/// </summary>
		/// <value> Zeitstempel der Erstellung der Abwicklungsantwort durch den Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundleReply()">
		/// </seealso>
		/// <seealso cref="TimestampForwardingRequest()">
		/// </seealso>
		virtual public Timestamp TimestampForwardingReply {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleReply.Forwarding;
			}
			
		}

		/// <summary> Liefert den im Auftragslaufzettel enthaltenen Zeitstempel vom Zeitpunkt des
		/// Eingangs einer positiven Bearbeitungsantwort vom Empfnger
		/// beim Intermedir.
		/// </summary>
		/// <value> Zeitstempel der Registrierung einer Empfangsbesttigung (Bearbeitungsantwort)
		/// durch den Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="TimestampForwardingRequest()">
		/// </seealso>
		virtual public Timestamp TimestampReceptionRequest {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleRequest.Reception;
			}
			
		}

		/// <summary> Liefert die Ergebnisse der Zertifikatsprfungen des Abwicklungsauftrags in
		/// Form von Inspection-Objekten, die im Laufzettel des Auftrags enthalten sind.
		/// </summary>
		/// <value> Prfergebnisse
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="InspectionsReply()">
		/// </seealso>
		virtual public Inspection[] InspectionsRequest {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleRequest.Inspections;
			}
			
		}

		/// <summary> Liefert die Ergebnisse der Zertifikatsprfungen des Abwicklungsauftrags in
		/// Form von Inspection-Objekten, die im Laufzettel der Antwort enthalten sind.
		/// </summary>
		/// <value> Prfergebnisse
		/// </value>
		/// <seealso cref="ProcessCardBundleReply()">
		/// </seealso>
		/// <seealso cref="InspectionsRequest()">
		/// </seealso>
		virtual public Inspection[] InspectionsReply {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleReply.Inspections;
			}
			
		}

		/// <summary> Liefert den im Auftragslaufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <value> Betreff der Auftragsnachricht
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="SubjectReply()">
		/// </seealso>
		virtual public System.String SubjectRequest {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleRequest.subject;
			}
			
		}

		/// <summary> Liefert den im Antwortlaufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <value> Betreff der Antwortnachricht
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="SubjectRequest()">
		/// </seealso>
		/// <deprecated> Im Zuge der Erweiterung des Interfaces (Version 1.0.3)</deprecated>
		virtual public System.String SubjectReply {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleReply.subject;
			}
		}

		/// <summary> Liefert den im Antwortlaufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <value> Betreff der Antwortnachricht
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="SubjectRequest">
		/// </seealso>
		public String Subject{
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleReply.Subject;
				 }
		}

		/// <summary> Liefert das Datum der letzten nderung des Auftragslaufzettels. Das Format
		/// entspricht dem XML-Schema nach <a href="http://www.w3.org/TR/xmlschema-2/#dateTime">
		/// http://www.w3.org/TR/xmlschema-2/#dateTime</a>.
		/// </summary>
		/// <value> Datum der letzten nderung.
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="RecentModificationReply()">
		/// </seealso>
		virtual public System.String RecentModificationRequest {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleRequest.recentModification;
			}
		}

		/// <summary> Liefert das Datum der letzten nderung des Antwortlaufzettels. Das Format
		/// entspricht dem XML-Schema nach <a href="http://www.w3.org/TR/xmlschema-2/#dateTime">
		/// http://www.w3.org/TR/xmlschema-2/#dateTime</a>.
		/// </summary>
		/// <value> Datum der letzten nderung.
		/// </value>
		/// <seealso cref="ProcessCardBundleRequest()">
		/// </seealso>
		/// <seealso cref="RecentModificationRequest()">
		/// </seealso>
		virtual public System.String RecentModificationReply {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleReply.recentModification;
			}
		}
/* pr mai
		/// <summary> Liefert die Message-ID der Nachricht (Auftrag).
		/// </summary>
		/// <value> Message-ID
		/// </value>
		virtual public System.String MessageId {
			get {
				return messageId;
			}
         		
		}
*/
		/// <summary> Liefert die Message-ID der Nachricht (Antwort).
		/// </summary>
		/// <value> Message-ID
		/// </value>
		/// <deprecated> Im Zuge der Erweiterung des Interfaces (Version 1.0.3) * de.osci.osci12.messagetypes.ContentPackageI ersetzt durch getMessageId() </deprecated>
		virtual public System.String MessageIdReply {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleReply.MessageId;
			}	
		}
		/// <summary> Liefert die Message-ID der Nachricht (Antwort).
		/// </summary>
		/// <value> Message-ID
		/// </value>
		virtual public System.String MessageId {
			get {
				if (processCardBundleRequest == null)return null;
				return processCardBundleReply.MessageId;
			}	
		}

		/// <summary> Liefert die in die Nachricht eingestellten Attachment als Attachment-Objekte.
		/// </summary>
		/// <value> enthaltene Attachment mit Inhaltsdaten.
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.Attachment">
		/// </seealso>
		override public Attachment[] Attachments {
			get {
				return base.Attachments;
			}
			
		}

		/// <summary> Liefert die in die Nachricht eingestellten (unverschlsselten) Inhaltsdaten als ContentContainer-Objekte.
		/// </summary>
		/// <value> enthaltene ContentContainer mit Inhaltsdaten.
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>
		override public ContentContainer[] ContentContainer {
			get {
				return base.ContentContainer;
			}
			
		}

		/// <summary> Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
		/// als EncryptedData-Objekte.
		/// </summary>
		/// <value> enthaltene EncryptedData-Objekt mit verschlsselten Inhaltsdaten.
		/// </value>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		override public EncryptedDataOSCI[] EncryptedData {
			get {
				return base.EncryptedData;
			}
			
		}

		/// <summary> Diese Methode registriert (statisch) eine OSCIDataSource-Implementierung,
		/// die fr die Speicherung aller eingehenden Nachrichten dieses Typs
		/// verwendet wird. Beim Empfang einer Nachricht wird vom registrierten
		/// OSCIDataSource-Objekt eine neue Instanz geholt (OSCIDataSource.newInstance())
		/// und die Nachricht (der eingehende Bytestrom) in deren OutputStream geschrieben.
		/// </summary>
		/// <seealso cref="de.osci.osci12.extinterfaces.OSCIDataSource">
		/// </seealso>
		public static OSCIDataSource InputDataSourceImpl {
			set {
				inputDataSourceImpl = value;
			}
		}

		/// <summary> Liefert die Instanz des registrierten OSCIDataSource-Objektes, welches
		/// fr die Speicherung der Nachricht beim Empfang verwendet wurde.
		/// Die Methode liefert null, wenn keine OSCIDataSource-Implementierung
		/// registriert wurde.
		/// </summary>
		/// <value> Instanz von OSCIDataSource
		/// </value>
		virtual public OSCIDataSource InputDataSource {
			get {
				return inputDataSource;
			}
			
		}

		new private static Log log;
		public ProcessCardBundle processCardBundleRequest;
		public ProcessCardBundle processCardBundleReply;
		private static OSCIDataSource inputDataSourceImpl;
		private OSCIDataSource inputDataSource;
		
		public ResponseToMediateDelivery(DialogHandler dh):base(dh)
		{
			messageType = RESPONSE_TO_MEDIATE_DELIVERY;
			// Weil in diesem Szenario die Rollen wechseln (der Orginator der Anfrage wird zum Addressee der Antwort
			// und umgekehrt) werden die Rollenobjekte umgebaut.
			de.osci.osci12.extinterfaces.crypto.Signer signer = null;
			de.osci.osci12.extinterfaces.crypto.Decrypter decrypter = null;
			if (dialogHandler.Client.hasSignaturePrivateKey())
			{
				signer = dialogHandler.Client.Signer;
			}
			 if (dialogHandler.Client.hasCipherPrivateKey())
			{
				decrypter = dialogHandler.Client.Decrypter;
			}
			addressee = new Addressee(signer, decrypter);
		}
		
		internal ResponseToMediateDelivery(MediateDelivery medDel, ResponseToProcessDelivery rspProcDel):base(medDel.dialogHandler)
		{
			messageType = RESPONSE_TO_MEDIATE_DELIVERY;
			addressee = rspProcDel.addressee;
			originator = rspProcDel.originator;
			ContentContainer[] con = rspProcDel.ContentContainer;
			for (int i = 0; i < con.Length; i++)
			{
				addContentContainer(con[i]);
			}
			EncryptedDataOSCI[] enc = rspProcDel.EncryptedData;
			for (int i = 0; i < enc.Length; i++)
			{
				addEncryptedData(enc[i]);
			}
			Attachment[] att = rspProcDel.Attachments;
			for (int i = 0; i < att.Length; i++)
			{
				addAttachment(att[i]);
			}
			Author[] auth = rspProcDel.OtherAuthors;
			for (int i = 0; i < auth.Length; i++)otherAutors[auth[i].id]=auth[i];
			de.osci.osci12.roles.Reader[] read = rspProcDel.OtherReaders;
			for (int i = 0; i < read.Length; i++)otherReaders[read[i].id]=read[i];
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			//    if (dialogHandler.getControlblock().getConversationID() == null)
			//    {
			//      dialogHandler.getControlblock().setConversationID(DialogHandler.idModule.createConversationID());
			//      DialogHandler.getDialogFinder().addDialog(dialogHandler);
			//    }
		}
		
		/// <summary> Fgt der Nachricht einen Inhaltsdatencontainer hinzu.
        /// <b>Hinweis: </b>ContentContainer-Objekte mit Attachments, die durch Entschlsselung 
        /// von EncryptedDataOSCI-Objekten gewonnen wurden, knnen hier nicht ohne weiteres
        /// hinzugefgt werden. In diesem Fall mssen die Attachments zunchst ausgelesen und 
        /// die Daten in neu angelegten Attachment-Objekten wieder hinzugefgt werden.
        /// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>
		public override void  addContentContainer(ContentContainer container)
		{
			base.addContentContainer(container);
		}
		
		/// <summary> Entfernt einen Inhaltsdatencontainer aus der Nachricht.
		/// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
		/// <seealso cref="addContentContainer">
		/// </seealso>
		public override void  removeContentContainer(ContentContainer container)
		{
			base.removeContentContainer(container);
		}
		
		
		/// <summary> Fgt der Nachricht einen EncryptedData-Eintrag mit verschlsselten
		/// Inhaltsdaten hinzu.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public override void  addEncryptedData(EncryptedDataOSCI encData)
		{
			base.addEncryptedData(encData);
		}
		
		/// <summary> Entfernt einen EncryptedData-Eintrag mit verschlsselten Daten
		/// aus der Nachricht.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="addEncryptedData">
		/// </seealso>
		/// <seealso cref="EncryptedDataOSCI"> 
		/// </seealso>
		public override void  removeEncryptedData(EncryptedDataOSCI encData)
		{
			base.removeEncryptedData(encData);
		}
		
		public override void  compose()
		{
			base.compose();
			System.String head = writeFeedBack();
			if (processCardBundleRequest != null)
			{
				System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
				processCardBundleRequest.writeXML(out_Renamed);
				if (processCardBundleReply != null)
				{
						processCardBundleReply.writeXML(out_Renamed);
				}
				char[] tmpChar;
				byte[] tmpByte;
				tmpByte = out_Renamed.GetBuffer();
				tmpChar = new char[out_Renamed.Length];
				System.Array.Copy(tmpByte, 0, tmpChar, 0, tmpChar.Length);
				head += new System.String(tmpChar);
			}
			osciH = new OsciH("responseToMediateDelivery", head);
			importAllCertificates();
			createNonIntermediaryCertificatesH();
			body = new Body(ContentContainer, EncryptedData);
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed)
		{
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null)
			{
				signatureHeader.writeXML(out_Renamed);
			}
			osciH.writeXML(out_Renamed);
			if (intermediaryCertificatesH != null)
			{
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			if (nonIntermediaryCertificatesH != null)
			{
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		static ResponseToMediateDelivery()
		{
			log = LogFactory.getLog(typeof(ResponseToMediateDelivery));
		}
		internal void setMessageIdRequest(String messageId) {
			this.messageId = messageId;
		}
	}
}