namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Zustellungsantwort</H4></p>
	/// Instanzen dieser Klasse werden als Antworten auf Zustellungsauftrge
	/// zurckgegeben. Das Nachrichtenobjekt enthlt eine Rckmeldung ber den Erfolg
	/// der Operation (getFeedback()) sowie ggf. den Laufzettel der
	/// Zustellung.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.StoreDelivery">
	/// </seealso>
	/// <seealso cref="ProcessCardBundle()">
	/// </seealso>
	public class ResponseToStoreDelivery:OSCIResponseTo
	{
		/// <summary> Diese Methode liefert den Laufzettel der Zustellung zurck oder null,
		/// wenn bei der Verarbeitung der Nachricht ein Fehler aufgetereten ist.
		/// Die Informationen im Laufzettel knnen auch direkt ber die einzelnen
		/// getX()-Methoden ausgewertet werden.
		/// </summary>
		/// <returns> den Laufzettel als ProcessCardBundle-Objekt, im Fehlerfall null
		/// </returns>
		/// <seealso cref="TimestampCreation()">
		/// </seealso>
		/// <seealso cref="Inspections()">
		/// </seealso>
		/// <seealso cref="Subject()">
		/// </seealso>
		/// <seealso cref="RecentModification()">
		/// </seealso>
		/// <seealso cref="MessageId()">
		/// </seealso>
		virtual public ProcessCardBundle ProcessCardBundle
		{
			get
			{
				return processCardBundle;
			}
			
		}
		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des Eingangs
		/// des Zustellungsauftrags beim Intermedir.
		/// </summary>
		/// <returns> Zeitstempel der Einreichung beim Intermedir
		/// </returns>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public Timestamp TimestampCreation
		{
			get
			{
				return processCardBundle.Creation;
			}
			
		}
		/// <summary> Liefert die Ergebnisse der Zertifikatsprfungen in Form von Inspection-Objekten,
		/// die im ProcessCardBundle-Objekt enthalten sind.
		/// </summary>
		/// <returns> inspections die Prfergebnisse
		/// </returns>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public Inspection[] Inspections
		{
			get
			{
				return processCardBundle.Inspections;
			}
			
		}
		/// <summary> Liefert den im Laufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <returns> den Betreff der Zustellung
		/// </returns>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public System.String Subject
		{
			get
			{
				return processCardBundle.subject;
			}
			
		}
		/// <summary> Liefert das Datum der letzten nderung des Laufzettels. Das Format
		/// entspricht dem XML-Schema nach http://www.w3.org/TR/xmlschema-2/#dateTime
		/// </summary>
		/// <returns> Datum der letzten nderung.
		/// </returns>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public System.String RecentModification
		{
			get
			{
				return processCardBundle.recentModification;
			}
			
		}
		/// <summary> Liefert die Message-ID der Nachricht.
		/// </summary>
		/// <returns> die Message-ID
		/// </returns>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public System.String MessageId
		{
			get
			{
				return processCardBundle.MessageId;
			}
			
		}
		/// <summary> Diese Methode registriert (statisch) eine OSCIDataSource-Implementierung,
		/// die fr die Speicherung aller eingehenden Nachrichten dieses Typs
		/// verwendet wird. Beim Empfang einer Nachricht wird vom registrierten
		/// OSCIDataSource-Objekt eine neue Instanz geholt (OSCIDataSource.newInstance())
		/// und die Nachricht (der eingehende Bytestrom) in deren OutputStream geschrieben.
		/// </summary>
		/// <param name="dataSource">
		/// </param>
		/// <seealso cref="de.osci.osci12.extinterfaces.OSCIDataSource">
		/// </seealso>
		public static OSCIDataSource InputDataSourceImpl
		{
			set
			{
				inputDataSourceImpl = value;
			}
			
		}
		/// <summary> Liefert die Instanz des registrierten OSCIDataSource-Objektes, welches
		/// fr die Speicherung der Nachricht beim Empfang verwendet wurde.
		/// Die Methode liefert null, wenn keine OSCIDataSource-Implementierung
		/// registriert wurde.
		/// </summary>
		/// <returns> Instanz von OSCIDataSource
		/// </returns>
		virtual public OSCIDataSource InputDataSource
		{
			get
			{
				return inputDataSource;
			}
			
		}
		new private static Log log;
		internal ProcessCardBundle processCardBundle;
		
		private static OSCIDataSource inputDataSourceImpl;
		private OSCIDataSource inputDataSource;
		
		internal ResponseToStoreDelivery(DialogHandler dh):this(dh, false)
		{
		}
		
		internal ResponseToStoreDelivery(DialogHandler dh, bool parser):base(dh)
		{
			messageType = RESPONSE_TO_STORE_DELIVERY;
			originator = ((Originator) dh.Client);
			
			if (!parser)
			{
				dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
				dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			}
			body = new Body("");
			body.setNSPrefixes(this);
		}
		
	
		public override void  compose()
		{
			base.compose();
			if (dialogHandler.Controlblock.Response == null)
			{
				throw new System.SystemException("Kein Response-Wert in DialogHandler eingestellt.");
			}
			if (dialogHandler.Controlblock.ConversationID == null)
			{
				throw new System.SystemException("Keine Conversation-ID in DialogHandler eingestellt.");
			}
			if (dialogHandler.Controlblock.SequenceNumber == - 1)
			{
				throw new System.SystemException("Kein Squenznummer in DialogHandler eingestellt.");
			}
			if (feedBack == null)
			{
				throw new System.SystemException("Kein Feedback eingestellt.");
			}
			if (processCardBundle == null)
			{
				osciH = new OsciH("responseToStoreDelivery", writeFeedBack());
			}
			else
			{
				System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
				processCardBundle.writeXML(out_Renamed);
				out_Renamed = new System.IO.MemoryStream();
				processCardBundle.writeXML(out_Renamed);
				char[] tmpChar;
				byte[] tmpByte;
				tmpByte = out_Renamed.GetBuffer();
				tmpChar = new char[out_Renamed.Length];
				System.Array.Copy(tmpByte, 0, tmpChar, 0, tmpChar.Length);
				osciH = new OsciH("responseToStoreDelivery", writeFeedBack() + new System.String(tmpChar));
			}
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed)
		{
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null)
			{
				signatureHeader.writeXML(out_Renamed);
			}
			osciH.writeXML(out_Renamed);
			if (intermediaryCertificatesH != null)
			{
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}

		static ResponseToStoreDelivery()
		{
			log = LogFactory.getLog(typeof(ResponseToStoreDelivery));
		}
	}
}