namespace de.osci.osci12.roles {
	
	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using de.osci.osci12.extinterfaces.crypto;
	using System.Web;
    using System.Security.Cryptography;
    using de.osci.osci12.common;
    using System.Runtime.CompilerServices;


	/// <summary> Diese Klasse ist die Superklasse aller OSCI-Rollenobjekte.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public abstract class Role {

		/// <summary> Ruft die Id, mit der in der OSCI-Nachricht das Verschlsselungszertifikat
		/// referenziert wird ab, oder legt diese fest. Die Id ist ein String, der aus dem Rollennamen,
		/// ist (in Kleinschreibung, alle Leerzeichen durch Unterstriche ersetzt).
		/// bei Author- und Reader-Objekten einer lfd. Nummer, der Funktionbezeichnung 
		/// (cipher) und dem SHA-1-Hashwert ber den Bytes des Zertifikats
		/// besteht 
		/// </summary>
		/// <value>Id des CipherCertificate-Elementes
		/// </value>
		/// <seealso cref="SignatureCertificateId()">
		/// </seealso>
		/// <exception cref="OSCIRoleException">wenn kein Verschlsselungszertifikat eingestellt ist
		/// </exception>
		virtual public System.String CipherCertificateId {
			get {
				try {
					if (!hasCipherCertificate())
						return id;
					if (cipherRefId != null)
						return cipherRefId;
					else
						return (id + "_cipher_" + hashCertificate(CipherCertificate));

					return (id + "_cipher_" + hashCertificate(CipherCertificate));
				}
				catch (Exception e) {
					return id;
				}
			}
		}

		protected String reverse(String s)
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			for (int i = s.Length-1; i>0;)
			{
				sb.Append(s[i-1]);
				sb.Append(s[i]);
				i -= 2;
			}
			return sb.ToString();
		}

		private string Encode(string oldValue) {
			string newValue = oldValue.Replace("\"", "'");
			newValue = System.Web.HttpUtility.UrlEncode(newValue);
			newValue = newValue.Replace("%2f", "/");
			return newValue;
		}
		
		/// <summary> Ruft die Id, mit der in der OSCI-Nachricht das Signaturzertifikat
		/// referenziert wird ab. Die Id ist ein String, der aus dem Rollennamen,
		/// bei Author- und Reader-Objekten einer lfd. Nummer, dem Namen des
		/// Zertifikatsherausgebers sowie der Seriennummer des Zertifikats zusammengesetzt
		/// ist (in Kleinschreibung, alle Leerzeichen durch Unterstriche ersetzt).
		/// </summary>
		/// <value>Id des SignatureCertificate-Elementes
		/// </value>
		/// <seealso cref="CipherCertificateId()">
		/// </seealso>
		/// <exception cref="OSCIRoleException">wenn kein Signaturzertifikat eingestellt ist
		/// </exception>
		virtual public System.String SignatureCertificateId {
			get {
				try {
					if (!hasSignatureCertificate()) 
						return id; 
					if (signatureRefId != null)
						return signatureRefId;
					else
						return (id + "_signature_" + hashCertificate(SignatureCertificate));
				} 
				catch (Exception e) {
					return id;
				}
			}
		}
		/** Stellt fest, ob ein Signatur-Privatschlssel (Signer) verfgbar ist. 
		* @return true -> Signer wurde gesetzt 
				   */ 
		public bool hasSignaturePrivateKey() 
		{ 
			return (signer != null); 
		} 
		/**    
			  * Stellt fest, ob ein Verschlsselungs-Privatschlssel (Decrypter) verfgbar ist. 
			  * @return true -> Decrypter wurde gesetzt 
			  */ 
		public bool hasCipherPrivateKey() 
		{ 
			return (decrypter != null); 
		} 
  
		/** 
		 * Stellt fest, ob ein Verschlsselungszertifikat (bzw. Decrypter) verfgbar ist. 
		 * @return true -> Verschlsselungszertifikat wurde gesetzt 
		 */ 
		public bool hasCipherCertificate() 
		{ 
			return ((decrypter != null) || (cipherCertificate != null)); 
		}

        /** 
         * Stellt fest, ob ein Signaturzertifikat (bzw. Signer) verfgbar ist. 
         * @return true -> Signaturzertifikat wurde gesetzt 
         */
        public bool hasSignatureCertificate()
		{
			return ((signer != null) || (signatureCertificate != null));
		}

        /**
         * Helfermethode fr die Berechnung des Hashwertes von Zertfifkaten.
         * Wird vom Anwender normalerweise nicht bentigt.
         *
         * @param cert zu hashendes Zertifkat
         *
         * @return hexadezimaler Hashwert
         *
         * @throws NoSuchAlgorithmException wenn Hashalgorithmus nicht verfgbar
         * @throws CertificateEncodingException bei Codierungsfehlern
         */

        [MethodImpl(MethodImplOptions.Synchronized)]
        public static String hashCertificate(X509Certificate cert) 
        {
            if (cert == null)
                return "no_hash_available";
			if (md == null)
                md = new SHA1Managed();
            return de.osci.osci12.encryption.Crypto.toHex(md.ComputeHash(cert.GetEncoded()));
	    }
		
		/// <summary> Ruft das eingestellte Signer-Objekt ab, oder null, wenn kein Objekt an den
		/// Konstruktor bergeben wurde, oder legt dieses fest.
		/// <p>Falls bereits ein Signaturzertifikat festgelegt wurde, wird dieses nach Aufruf 
		/// dieser Methode ignoriert. Alle weiteren Aufrufe von SignatureCertificate() liefern 
		/// dann das Zertifkat des Signer-Objektes zurck. Signer(null) lscht das Signer-Objekt.</p>
		/// </summary>
		/// <value>Signer-Objekt
		/// </value>
		/// <exception cref="OSCIRoleException">wenn kein Signer gesetzt wurde
		/// </exception>
		virtual public Signer Signer {
			get {
				if (signer == null) {
					throw new OSCIRoleException("Kein Signer-Objekt fr Objekt " + id + " eingestellt.", "no_signer_" + id);
				}
				return signer;
			}
			set {
				this.signer = value;
			}	
		}

		/// <summary>Ruft das eingestellte Decrypter-Objekt ab, oder null, wenn kein Objekt an den
		/// Konstruktor bergeben wurde, oder legt dieses fest.
		/// </summary>
		/// <value>Decrypter-Objekt
		/// </value>
		/// <exception cref="OSCIRoleException">wenn kein Decrypter gesetzt wurde
		/// </exception>
		virtual public Decrypter Decrypter {
			get {
				if (decrypter == null) {
					throw new OSCIRoleException("Kein Decrypter-Objekt fr Objekt " + id + " eingestellt.", "no_decrypter_" + id);
				}
				return decrypter;
			}
			set {
				this.decrypter = value;
			}	
		}

		/// <summary>Ruft das eingestellte Signaturzertifikat ab, oder legt dieses fest.
		/// </summary>
		/// <value> Signaturzertifikat
		/// </value>
		/// <exception cref="OSCIRoleException">wenn kein Signaturzertifikat eingestellt ist.
		/// </exception>
		virtual public X509Certificate SignatureCertificate {
			get {
				if (signer == null && signatureCertificate == null) {
					throw new OSCIRoleException("Es wurde kein Signaturzertifikate fr das Rollenobjekt vom Typ " + id + " eingestellt.", "no_signature_cert_" + id);
				} else if (signer == null) {
					return signatureCertificate;
				} else {
					return signer.Certificate;
				}
			}
			set {
				this.signatureCertificate = value;
			}	
		}

		/// <summary> Ruft das eingestellte Verschlsselungszertifikat ab, oder legt dieses fest.
		/// </summary>
		/// <value> Verschlsselungszertifikat
		/// </value>
		/// <exception cref="OSCIRoleException">wenn kein Verschlsselungszertifikat eingestellt ist.
		/// </exception>
		virtual public X509Certificate CipherCertificate {
			get {
				if (decrypter == null && cipherCertificate == null) 
				{
					throw new OSCIRoleException("Es wurde kein Verschlsselungszertifikate fr das Rollenobjekt vom Typ " + id + " eingestellt.", "no_cipher_cert_" + id);
				} else if (decrypter == null) {
					return cipherCertificate;
				} else {
					return decrypter.Certificate;
				}
			}
			set {
				this.cipherCertificate = value;
			}
		}
		
		/// <summary> Liefert (ab Version 1.3) den Signaturalgorithmus,
        /// der von der Methode Signer.getAlgorithm() zurckgeliefert wird. 
        /// Liefert diese Methode null oder ist sie nicht implementiert, 
        /// wird der im {@link DialogHandler} eingestellte Default-Signaturalgorithmus 
        /// zurckgegeben.</summary>
        /// <returns>den Identifier des Algorithmus</returns>

		virtual public System.String SignatureAlgorithm {
			get {
            	  if (!hasSignaturePrivateKey())
		            throw new Exception("Kein Signer-Objekt gesetzt.");

	                try
	                {
	    	            signatureAlgorithm = signer.getAlgorithm();
	    	            if (signatureAlgorithm == null)
		    	            signatureAlgorithm = DialogHandler.SignatureAlgorithm;	    		
	                }
	                catch (Exception ame)
            	    {
	    	            log.warn("No implementaion of Signer.getAlgorithm() found in Role '" + id  
	    			        + "', probably old Signer implementaion (< 1.3) in use. Defaulting to " 
	    			        + DialogHandler.SignatureAlgorithm);
	    	            signatureAlgorithm = DialogHandler.SignatureAlgorithm;
	                }

	                return signatureAlgorithm;
			}
		}

		private static Log log;
		private static SHA1Managed md;
		protected internal Signer signer = null;
		protected internal Decrypter decrypter = null;
		protected internal X509Certificate signatureCertificate = null;
		protected internal X509Certificate cipherCertificate = null;
		/** Referenz-Ids, sollten von Anwendungen nicht gesetzt werden. */ 
		public String cipherRefId, signatureRefId; 
  
		
		/// <summary> Interne Referenz-Id, sollte von Anwendungen nicht gesetzt werden.
		/// </summary>
		public System.String id;

		protected internal System.String signatureAlgorithm = de.osci.osci12.common.Constants.SIGNATURE_ALGORITHM_RSA_SHA256;
		
		protected internal Role() {
			id = GetType().FullName.Substring(GetType().FullName.LastIndexOf((System.Char) '.') + 1).ToLower().Replace('$', '_');
		}

		static Role() 
        {
			log = LogFactory.getLog(typeof(Role));
		}
	}
}