namespace de.osci.osci12.samples.impl.crypto {
	
using System;
using de.osci.osci12.common;
using de.osci.osci12.extinterfaces.crypto;
using de.osci.osci12.dotcommon;
using org.apache.commons.logging;
using System.Security.Cryptography;
//using System.Security.Cryptography.X509Certificates;
using java.security.cert;
using System.IO;
using System.Collections;

using Org.BouncyCastle.Crypto.Signers;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Pkcs;
using Org.BouncyCastle.Math;
using Org.BouncyCastle.Crypto.Generators;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Security;
using Org.BouncyCastle.X509;


	/// <summary><p>Diese Klasse ist eine Beispiel-Implementierung der abstrakten Signer-Klasse.
	/// Fr die Verwendung wird ein PKCS#12-Keystore in Form einer *.p12-Datei bentigt.
	/// Die Implementierung ist fr Testzwecke bestimmt, sie greift auf den ersten
	/// verfgbaren Alias zu. Die PIN fr dessen Privatschlssel muss die gleiche sein
	/// wie die des Keystores.</p>
	/// <p>Diese einfache Implementierung hlt die PIN des Keystores als Character-Array
	/// im Arbeitsspeicher, sie wird als String bergeben. Es wird Anwendern empfohlen,
	/// eigene Implementierungen zu schreiben, die die PIN in der Methode sign(...)
	/// abfragen und nach Gebrauch wieder lschen oder anderweitig fr ein sicheres
	/// Pin-Cashing zu sorgen.</p> 
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.extinterfaces.crypto.Signer">
	/// </seealso>
    public class PKCS12Signer : de.osci.osci12.extinterfaces.crypto.Signer
    {
        private static Log log = LogFactory.getLog(typeof(PKCS12Signer));
        private java.security.cert.X509Certificate cert;
        private String cipherAlgorithm;
        private AsymmetricKeyParameter pKey;


        /// <summary> Liefert den Namen des Herstellers.
        /// </summary>
        /// <value> Herstellername
        /// </value>
        public override String Vendor
        {
            get
            {
                return "BOS";
            }
        }

        /// <summary> Liefert die Versionsnummer.
        /// </summary>
        /// <value> Versionsnummer
        /// </value>
        public override String Version
        {
            get
            {
                return "1.5";
            }
        }

        /// <summary> Liefert das Signaturzertifikat.
        /// </summary>
        /// <value> Verschlsselungszertifikat
        /// </value>
        public override java.security.cert.X509Certificate Certificate
        {
            get
            {
                return cert;
            }
        }


        public PKCS12Signer(String p12_fileName, String pin)
        {
            System.IO.Stream pfile = new FileStream(p12_fileName, FileMode.Open);
            Pkcs12Store ks = new Pkcs12Store(pfile, pin.ToCharArray());
            pfile.Close();
            IEnumerator al = ks.Aliases.GetEnumerator();
            while (al.MoveNext())
            {
                if (ks.IsKeyEntry((String)al.Current))
                {
                    pKey = ks.GetKey((String)al.Current).Key;
                    cert = new java.security.cert.X509Certificate(ks.GetCertificate((String)al.Current).Certificate.GetEncoded());
                    if (pKey is ECPrivateKeyParameters)
                        cipherAlgorithm = "EC";
                    else if (pKey is RsaPrivateCrtKeyParameters)
                        cipherAlgorithm = "RSA";
                    else
                        throw new ArgumentException("Unsupported algorithm found, key is of type " + pKey.GetType().ToString());
                    break;
                }
            }
            if (pKey == null)
                throw new ArgumentException("No private key found in keystore !");
        }


        /// <summary> Signiert das bergebene Byte-Array.
        /// </summary>
        /// <param name="parm1"> die zu signierenden Daten
        /// </param>
        /// <param name="algorithm"> Algorithmus
        /// </param>
        /// <returns> die Signatur
        /// </returns>
        public override byte[] sign(byte[] data, String algorithm)
        {

            String digAlg = null;
            if (algorithm.EndsWith("sha256"))
                digAlg = "SHA-256";
            else if (algorithm.EndsWith("sha512"))
                digAlg = "SHA-512";
            else if (algorithm.EndsWith("sha1"))
                digAlg = "SHA-1";

            if (cipherAlgorithm.Equals("EC"))
            {
                ECDsaSigner signer = new ECDsaSigner();
                signer.Init(true, pKey);
                byte[] digest = DigestUtilities.CalculateDigest(digAlg, data);
                BigInteger[] sigBigInts = signer.GenerateSignature(digest);
                int len = (((ECPrivateKeyParameters)pKey).Parameters.N.BitLength + 7) / 8;
                byte[] s = new byte[len * 2];
                byte[] s0 = sigBigInts[0].ToByteArray();
                Array.Copy(s0, s0.Length - len, s, 0, len);
                byte[] s1 = sigBigInts[1].ToByteArray();
                Array.Copy(s1, s1.Length - len, s, len, len);
                return s;
            }
            else
            {
                IDigest id = DigestUtilities.GetDigest(digAlg);
                RsaDigestSigner rsads = new RsaDigestSigner(id);
                rsads.Init(true, pKey);
                rsads.BlockUpdate(data, 0, data.Length);
                return rsads.GenerateSignature();
            }

        }

        public override String getAlgorithm()
        {
            String algo = DialogHandler.SignatureAlgorithm;

            try
            {
                if (algo.EndsWith("sha256") && cipherAlgorithm.Equals("EC"))
                    algo = Constants.SIGNATURE_ALGORITHM_ECDSA_SHA256;
                else if (algo.EndsWith("sha512") && cipherAlgorithm.Equals("EC"))
                    algo = Constants.SIGNATURE_ALGORITHM_ECDSA_SHA512;
                else if (algo.EndsWith("sha256") && cipherAlgorithm.Equals("RSA"))
                    algo = Constants.SIGNATURE_ALGORITHM_RSA_SHA256;
                else if (algo.EndsWith("sha512") && cipherAlgorithm.Equals("RSA"))
                    algo = Constants.SIGNATURE_ALGORITHM_RSA_SHA512;
            }
            catch (Exception e)
            {
                log.error("Problem getting private key type, return to default", e);
            }
  
            return algo;
        }
    }
}