namespace org.apache.commons.logging
{
	using System;
	using System.IO;
	
	/// <summary> Dieser Klasse dient der Erzeugung von Log-Ausgaben.
	/// Der Loglevel, der global fr alle Log-Objekte voreingestellt wird,
	/// kann ber eine Umgebungsvariable "OSCI_LIB_DEBUG_LEVEL" gesetzt werden.
	/// Auerdem kann durch Setzen einer weiteren Umgebungsvariable
	/// "OSCI_LIB_DEBUG_FILE" bewirkt werden, dass alle Log-Ausgaben in eine Datei geschrieben
	/// werden. Dadurch wird allerdings der Programmablauf stark verlangsamt.
	/// 
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class Log
	{
		public const int LEVEL_OFF = 0;
		public const int LEVEL_TRACE = 1;
		public const int LEVEL_DEBUG = 2;
		public const int LEVEL_INFO = 3;
		public const int LEVEL_WARN = 4;
		public const int LEVEL_ERROR = 5;
		public const int LEVEL_FATAL = 6;

		private static String[] level_names = new String[]{"OFF", "TRACE", "DEBUG", "INFO", "WARN", "ERROR", "FATAL"};

		private static bool initialized = false;		
		private static Stream file;		
		private static String filename;		
		private String message;
		private byte[] array;

		/// <summary>
		/// Enthlt den aktuellen Log-Level. Default ist LEVEL_ERROR. Durch Zuweisung im Code kann der
		/// Loglevel fr einzelne Klassen gendert werden.
		/// </summary>
//        public int level = LEVEL_TRACE;
//        private static int global_level = LEVEL_TRACE;
        public int level = LEVEL_ERROR;
        private static int global_level = LEVEL_ERROR;

		public bool TraceEnabled {get {return (level <= LEVEL_TRACE);}}
		public bool DebugEnabled {get {return (level <= LEVEL_DEBUG);}}
		public bool InfoEnabled {get {return (level <= LEVEL_INFO);}}
		public bool WarnEnabled {get {return (level <= LEVEL_WARN);}}
		public bool ErrorEnabled {get {return (level <= LEVEL_ERROR);}}
		public bool FatalEnabled {get {return (level <= LEVEL_FATAL);}}

		private String type;

		public Log(System.Type type)
		{
			this.type = type.ToString();
			level = global_level;
			init();
		}

		public Log(String type)
		{
			this.type = type;
			level = global_level;
			init();
		}

		private void init()
		{
			if (initialized)
				return;

			try
			{
				filename = System.Environment.GetEnvironmentVariable("OSCI_LIB_DEBUG_FILE");
				if (filename != null)
					file = new System.IO.FileStream(filename, System.IO.FileMode.Create, System.IO.FileAccess.Write);
			}
			catch (Exception e)
			{
				error("Problem", e);
			}

			String lv = System.Environment.GetEnvironmentVariable("OSCI_LIB_DEBUG_LEVEL");
			if (lv != null)
			{
				lv = lv.ToUpper();
				int i;
				for (i=0; i<level_names.Length; i++)
				{
					if (lv.Equals(level_names[i]))
					{						
						global_level = i;
						level = i;
						break;
					}
				}	
				if (i == level_names.Length)
					error("Invalid loglevel: " + lv);
			}

			initialized = true;

			if (file != null)
			{
				array = SupportClass.ToByteArray("File-Logging initialized on level " + level_names[level] + "\n\n");
				file.Write(array, 0, array.Length);
				file.Close();
			}
			else
			{
				System.Console.WriteLine("Console-Logging initialized on level " + level_names[level] + "\n");
			}
		}

		private void publish(int loglevel, Object o, Exception e)
		{
            if (level == 0)
                return;
			if (loglevel >= level)
			{
				message = level_names[loglevel] + " - " + o;
				if (e != null)
					message += "\n"  + e.Message + "\n" + e.StackTrace; 

				if (file != null)
				{
					lock(this)
					{
						file = new System.IO.FileStream(filename, System.IO.FileMode.Append, System.IO.FileAccess.Write);
						array = SupportClass.ToByteArray(message + "\n");
						file.Write(array, 0, array.Length);
						file.Close();
					}
				}
				else
					System.Console.WriteLine(message);
			}

		}

		public void trace(System.Object object_Renamed)
		{
			publish(LEVEL_TRACE, object_Renamed, null);
		}

		public void trace(System.Object object_Renamed, System.Exception throwable)
		{
			publish(LEVEL_TRACE, object_Renamed, throwable);
		}
		
		public void debug(System.Object object_Renamed)
		{
			publish(LEVEL_DEBUG, object_Renamed, null);
		}

		public void debug(System.Object object_Renamed, System.Exception throwable)
		{
			publish(LEVEL_DEBUG, object_Renamed, throwable);
		}

		public void info(System.Object object_Renamed)
		{
			publish(LEVEL_INFO, object_Renamed, null);
		}

		public void info(System.Object object_Renamed, System.Exception throwable)
		{
			publish(LEVEL_INFO, object_Renamed, throwable);
		}

		public void warn(System.Object object_Renamed)
		{
			publish(LEVEL_WARN, object_Renamed, null);
		}

		public void warn(System.Object object_Renamed, System.Exception throwable)
		{
			publish(LEVEL_WARN, object_Renamed, throwable);
		}

		public void error(System.Object object_Renamed)
		{
			publish(LEVEL_ERROR, object_Renamed, null);
		}
		
		public void error(System.Object object_Renamed, System.Exception throwable)
		{
			publish(LEVEL_ERROR, object_Renamed, throwable);
		}

		public void fatal(System.Object object_Renamed)
		{
			publish(LEVEL_FATAL, object_Renamed, null);
		}

		public void fatal(System.Object object_Renamed, System.Exception throwable)
		{
			publish(LEVEL_FATAL, object_Renamed, throwable);
		}
	}
}